# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    The version of the OpenAPI document: 1.113
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401
import sys  # noqa: F401

# python 2 and python 3 compatibility library
import six

from onshape_client.oas.api_client import ApiClient
from onshape_client.oas.exceptions import ApiTypeError, ApiValueError
from onshape_client.oas.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    int,
    none_type,
    str,
    validate_and_convert_types,
)
from onshape_client.oas.models import bt_workflowable_test_object_info
from onshape_client.oas.models import bt_app_element_basic_info
from onshape_client.oas.models import bt_app_associative_data_info_array
from onshape_client.oas.models import bt_list_response_bt_insertable_info
from onshape_client.oas.models import bt_metadata_schema_info
from onshape_client.oas.models import bt_list_response_bt_metadata_property_summary_info
from onshape_client.oas.models import bt_metadata_property_info
from onshape_client.oas.models import bt_bounding_box_info
from onshape_client.oas.models import bt_update_workflowable_test_object_params


class DefaultApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __create_workflowable_test_object(self, wfid, **kwargs):
            """create_workflowable_test_object  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.create_workflowable_test_object(wfid, async_req=True)
            >>> result = thread.get()

            Args:
                wfid (str):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_workflowable_test_object_info.BTWorkflowableTestObjectInfo
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["wfid"] = wfid
            return self.call_with_http_info(**kwargs)

        self.create_workflowable_test_object = Endpoint(
            settings={
                "response_type": (
                    bt_workflowable_test_object_info.BTWorkflowableTestObjectInfo,
                ),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/workflowabletestobject/testobject/{wfid}",
                "operation_id": "create_workflowable_test_object",
                "http_method": "POST",
                "servers": [],
            },
            params_map={
                "all": ["wfid",],
                "required": ["wfid",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {"wfid": (str,),},
                "attribute_map": {"wfid": "wfid",},
                "location_map": {"wfid": "path",},
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__create_workflowable_test_object,
        )

        def __delete_associative_data(self, did, eid, wvm, wvmid, **kwargs):
            """delete_associative_data  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.delete_associative_data(did, eid, wvm, wvmid, async_req=True)
            >>> result = thread.get()

            Args:
                did (str):
                eid (str):
                wvm (str):
                wvmid (str):

            Keyword Args:
                transaction_id (str): [optional] if omitted the server will use the default value of ''
                parent_change_id (str): [optional] if omitted the server will use the default value of ''
                associative_data_id ([str]): [optional]
                element_id (str): [optional] if omitted the server will use the default value of ''
                view_id (str): [optional] if omitted the server will use the default value of ''
                microversion_id (str): [optional] if omitted the server will use the default value of ''
                document_microversion (str): [optional] if omitted the server will use the default value of ''
                deterministic_id (str): [optional] if omitted the server will use the default value of ''
                feature_id (str): [optional] if omitted the server will use the default value of ''
                entity_id (str): [optional] if omitted the server will use the default value of ''
                occurrence_id (str): [optional] if omitted the server will use the default value of ''
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_app_element_basic_info.BTAppElementBasicInfo
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["did"] = did
            kwargs["eid"] = eid
            kwargs["wvm"] = wvm
            kwargs["wvmid"] = wvmid
            return self.call_with_http_info(**kwargs)

        self.delete_associative_data = Endpoint(
            settings={
                "response_type": (bt_app_element_basic_info.BTAppElementBasicInfo,),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/appelements/d/{did}/{wvm}/{wvmid}/e/{eid}/associativedata",
                "operation_id": "delete_associative_data",
                "http_method": "DELETE",
                "servers": [],
            },
            params_map={
                "all": [
                    "did",
                    "eid",
                    "wvm",
                    "wvmid",
                    "transaction_id",
                    "parent_change_id",
                    "associative_data_id",
                    "element_id",
                    "view_id",
                    "microversion_id",
                    "document_microversion",
                    "deterministic_id",
                    "feature_id",
                    "entity_id",
                    "occurrence_id",
                ],
                "required": ["did", "eid", "wvm", "wvmid",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "did": (str,),
                    "eid": (str,),
                    "wvm": (str,),
                    "wvmid": (str,),
                    "transaction_id": (str,),
                    "parent_change_id": (str,),
                    "associative_data_id": ([str],),
                    "element_id": (str,),
                    "view_id": (str,),
                    "microversion_id": (str,),
                    "document_microversion": (str,),
                    "deterministic_id": (str,),
                    "feature_id": (str,),
                    "entity_id": (str,),
                    "occurrence_id": (str,),
                },
                "attribute_map": {
                    "did": "did",
                    "eid": "eid",
                    "wvm": "wvm",
                    "wvmid": "wvmid",
                    "transaction_id": "transactionId",
                    "parent_change_id": "parentChangeId",
                    "associative_data_id": "associativeDataId",
                    "element_id": "elementId",
                    "view_id": "viewId",
                    "microversion_id": "microversionId",
                    "document_microversion": "documentMicroversion",
                    "deterministic_id": "deterministicId",
                    "feature_id": "featureId",
                    "entity_id": "entityId",
                    "occurrence_id": "occurrenceId",
                },
                "location_map": {
                    "did": "path",
                    "eid": "path",
                    "wvm": "path",
                    "wvmid": "path",
                    "transaction_id": "query",
                    "parent_change_id": "query",
                    "associative_data_id": "query",
                    "element_id": "query",
                    "view_id": "query",
                    "microversion_id": "query",
                    "document_microversion": "query",
                    "deterministic_id": "query",
                    "feature_id": "query",
                    "entity_id": "query",
                    "occurrence_id": "query",
                },
                "collection_format_map": {"associative_data_id": "multi",},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__delete_associative_data,
        )

        def __get_associative_data(self, did, wvm, wvmid, eid, **kwargs):
            """get_associative_data  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_associative_data(did, wvm, wvmid, eid, async_req=True)
            >>> result = thread.get()

            Args:
                did (str):
                wvm (str):
                wvmid (str):
                eid (str):

            Keyword Args:
                transaction_id (str): [optional] if omitted the server will use the default value of ''
                change_id (str): [optional] if omitted the server will use the default value of ''
                associative_data_id ([str]): [optional]
                element_id (str): [optional] if omitted the server will use the default value of ''
                view_id (str): [optional] if omitted the server will use the default value of ''
                microversion_id (str): [optional] if omitted the server will use the default value of ''
                document_microversion (str): [optional] if omitted the server will use the default value of ''
                deterministic_id (str): [optional] if omitted the server will use the default value of ''
                feature_id (str): [optional] if omitted the server will use the default value of ''
                entity_id (str): [optional] if omitted the server will use the default value of ''
                occurrence_id (str): [optional] if omitted the server will use the default value of ''
                return_id_tags (bool): [optional] if omitted the server will use the default value of False
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_app_associative_data_info_array.BTAppAssociativeDataInfoArray
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["did"] = did
            kwargs["wvm"] = wvm
            kwargs["wvmid"] = wvmid
            kwargs["eid"] = eid
            return self.call_with_http_info(**kwargs)

        self.get_associative_data = Endpoint(
            settings={
                "response_type": (
                    bt_app_associative_data_info_array.BTAppAssociativeDataInfoArray,
                ),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/appelements/d/{did}/{wvm}/{wvmid}/e/{eid}/associativedata",
                "operation_id": "get_associative_data",
                "http_method": "GET",
                "servers": [],
            },
            params_map={
                "all": [
                    "did",
                    "wvm",
                    "wvmid",
                    "eid",
                    "transaction_id",
                    "change_id",
                    "associative_data_id",
                    "element_id",
                    "view_id",
                    "microversion_id",
                    "document_microversion",
                    "deterministic_id",
                    "feature_id",
                    "entity_id",
                    "occurrence_id",
                    "return_id_tags",
                ],
                "required": ["did", "wvm", "wvmid", "eid",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "did": (str,),
                    "wvm": (str,),
                    "wvmid": (str,),
                    "eid": (str,),
                    "transaction_id": (str,),
                    "change_id": (str,),
                    "associative_data_id": ([str],),
                    "element_id": (str,),
                    "view_id": (str,),
                    "microversion_id": (str,),
                    "document_microversion": (str,),
                    "deterministic_id": (str,),
                    "feature_id": (str,),
                    "entity_id": (str,),
                    "occurrence_id": (str,),
                    "return_id_tags": (bool,),
                },
                "attribute_map": {
                    "did": "did",
                    "wvm": "wvm",
                    "wvmid": "wvmid",
                    "eid": "eid",
                    "transaction_id": "transactionId",
                    "change_id": "changeId",
                    "associative_data_id": "associativeDataId",
                    "element_id": "elementId",
                    "view_id": "viewId",
                    "microversion_id": "microversionId",
                    "document_microversion": "documentMicroversion",
                    "deterministic_id": "deterministicId",
                    "feature_id": "featureId",
                    "entity_id": "entityId",
                    "occurrence_id": "occurrenceId",
                    "return_id_tags": "returnIdTags",
                },
                "location_map": {
                    "did": "path",
                    "wvm": "path",
                    "wvmid": "path",
                    "eid": "path",
                    "transaction_id": "query",
                    "change_id": "query",
                    "associative_data_id": "query",
                    "element_id": "query",
                    "view_id": "query",
                    "microversion_id": "query",
                    "document_microversion": "query",
                    "deterministic_id": "query",
                    "feature_id": "query",
                    "entity_id": "query",
                    "occurrence_id": "query",
                    "return_id_tags": "query",
                },
                "collection_format_map": {"associative_data_id": "multi",},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__get_associative_data,
        )

        def __get_latest_in_document(self, did, **kwargs):
            """insertables for a document  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_latest_in_document(did, async_req=True)
            >>> result = thread.get()

            Args:
                did (str):

            Keyword Args:
                beta_capability_ids ([str]): [optional]
                include_parts (bool): [optional] if omitted the server will use the default value of False
                include_surfaces (bool): [optional] if omitted the server will use the default value of False
                include_wires (bool): [optional] if omitted the server will use the default value of False
                include_sketches (bool): [optional] if omitted the server will use the default value of False
                include_reference_features (bool): [optional] if omitted the server will use the default value of False
                include_assemblies (bool): [optional] if omitted the server will use the default value of False
                include_features (bool): [optional] if omitted the server will use the default value of False
                include_feature_studios (bool): [optional] if omitted the server will use the default value of False
                include_part_studios (bool): [optional] if omitted the server will use the default value of False
                include_blobs (bool): [optional] if omitted the server will use the default value of False
                include_meshes (bool): [optional] if omitted the server will use the default value of False
                include_flattened_bodies (bool): [optional] if omitted the server will use the default value of False
                allowed_blob_mime_types (str): [optional] if omitted the server will use the default value of ''
                max_feature_script_version (int): [optional] if omitted the server will use the default value of 0
                include_applications (bool): [optional] if omitted the server will use the default value of False
                allowed_application_mime_types (str): [optional] if omitted the server will use the default value of ''
                include_composite_parts (bool): [optional] if omitted the server will use the default value of False
                include_fs_tables (bool): [optional] if omitted the server will use the default value of False
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_list_response_bt_insertable_info.BTListResponseBTInsertableInfo
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["did"] = did
            return self.call_with_http_info(**kwargs)

        self.get_latest_in_document = Endpoint(
            settings={
                "response_type": (
                    bt_list_response_bt_insertable_info.BTListResponseBTInsertableInfo,
                ),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/insertables/d/{did}/latest",
                "operation_id": "get_latest_in_document",
                "http_method": "GET",
                "servers": [],
            },
            params_map={
                "all": [
                    "did",
                    "beta_capability_ids",
                    "include_parts",
                    "include_surfaces",
                    "include_wires",
                    "include_sketches",
                    "include_reference_features",
                    "include_assemblies",
                    "include_features",
                    "include_feature_studios",
                    "include_part_studios",
                    "include_blobs",
                    "include_meshes",
                    "include_flattened_bodies",
                    "allowed_blob_mime_types",
                    "max_feature_script_version",
                    "include_applications",
                    "allowed_application_mime_types",
                    "include_composite_parts",
                    "include_fs_tables",
                ],
                "required": ["did",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "did": (str,),
                    "beta_capability_ids": ([str],),
                    "include_parts": (bool,),
                    "include_surfaces": (bool,),
                    "include_wires": (bool,),
                    "include_sketches": (bool,),
                    "include_reference_features": (bool,),
                    "include_assemblies": (bool,),
                    "include_features": (bool,),
                    "include_feature_studios": (bool,),
                    "include_part_studios": (bool,),
                    "include_blobs": (bool,),
                    "include_meshes": (bool,),
                    "include_flattened_bodies": (bool,),
                    "allowed_blob_mime_types": (str,),
                    "max_feature_script_version": (int,),
                    "include_applications": (bool,),
                    "allowed_application_mime_types": (str,),
                    "include_composite_parts": (bool,),
                    "include_fs_tables": (bool,),
                },
                "attribute_map": {
                    "did": "did",
                    "beta_capability_ids": "betaCapabilityIds",
                    "include_parts": "includeParts",
                    "include_surfaces": "includeSurfaces",
                    "include_wires": "includeWires",
                    "include_sketches": "includeSketches",
                    "include_reference_features": "includeReferenceFeatures",
                    "include_assemblies": "includeAssemblies",
                    "include_features": "includeFeatures",
                    "include_feature_studios": "includeFeatureStudios",
                    "include_part_studios": "includePartStudios",
                    "include_blobs": "includeBlobs",
                    "include_meshes": "includeMeshes",
                    "include_flattened_bodies": "includeFlattenedBodies",
                    "allowed_blob_mime_types": "allowedBlobMimeTypes",
                    "max_feature_script_version": "maxFeatureScriptVersion",
                    "include_applications": "includeApplications",
                    "allowed_application_mime_types": "allowedApplicationMimeTypes",
                    "include_composite_parts": "includeCompositeParts",
                    "include_fs_tables": "includeFSTables",
                },
                "location_map": {
                    "did": "path",
                    "beta_capability_ids": "query",
                    "include_parts": "query",
                    "include_surfaces": "query",
                    "include_wires": "query",
                    "include_sketches": "query",
                    "include_reference_features": "query",
                    "include_assemblies": "query",
                    "include_features": "query",
                    "include_feature_studios": "query",
                    "include_part_studios": "query",
                    "include_blobs": "query",
                    "include_meshes": "query",
                    "include_flattened_bodies": "query",
                    "allowed_blob_mime_types": "query",
                    "max_feature_script_version": "query",
                    "include_applications": "query",
                    "allowed_application_mime_types": "query",
                    "include_composite_parts": "query",
                    "include_fs_tables": "query",
                },
                "collection_format_map": {"beta_capability_ids": "multi",},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__get_latest_in_document,
        )

        def __get_metadata_schema(self, object_type, **kwargs):
            """get_metadata_schema  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_metadata_schema(object_type, async_req=True)
            >>> result = thread.get()

            Args:
                object_type (int):

            Keyword Args:
                owner_id (str): [optional]
                document_id (str): [optional]
                owner_type (int): [optional] if omitted the server will use the default value of 1
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_metadata_schema_info.BTMetadataSchemaInfo
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["object_type"] = object_type
            return self.call_with_http_info(**kwargs)

        self.get_metadata_schema = Endpoint(
            settings={
                "response_type": (bt_metadata_schema_info.BTMetadataSchemaInfo,),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/metadataschema",
                "operation_id": "get_metadata_schema",
                "http_method": "GET",
                "servers": [],
            },
            params_map={
                "all": ["object_type", "owner_id", "document_id", "owner_type",],
                "required": ["object_type",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "object_type": (int,),
                    "owner_id": (str,),
                    "document_id": (str,),
                    "owner_type": (int,),
                },
                "attribute_map": {
                    "object_type": "objectType",
                    "owner_id": "ownerId",
                    "document_id": "documentId",
                    "owner_type": "ownerType",
                },
                "location_map": {
                    "object_type": "query",
                    "owner_id": "query",
                    "document_id": "query",
                    "owner_type": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__get_metadata_schema,
        )

        def __get_properties(self, **kwargs):
            """get_properties  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_properties(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                schema_id (str): [optional]
                owner_id (str): [optional]
                document_id (str): [optional]
                owner_type (int): [optional] if omitted the server will use the default value of 1
                object_type (int): [optional]
                strict (bool): [optional] if omitted the server will use the default value of False
                active_only (bool): [optional] if omitted the server will use the default value of False
                offset (int): [optional] if omitted the server will use the default value of 0
                limit (int): [optional] if omitted the server will use the default value of 200
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_list_response_bt_metadata_property_summary_info.BTListResponseBTMetadataPropertySummaryInfo
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            return self.call_with_http_info(**kwargs)

        self.get_properties = Endpoint(
            settings={
                "response_type": (
                    bt_list_response_bt_metadata_property_summary_info.BTListResponseBTMetadataPropertySummaryInfo,
                ),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/metadataschema/properties",
                "operation_id": "get_properties",
                "http_method": "GET",
                "servers": [],
            },
            params_map={
                "all": [
                    "schema_id",
                    "owner_id",
                    "document_id",
                    "owner_type",
                    "object_type",
                    "strict",
                    "active_only",
                    "offset",
                    "limit",
                ],
                "required": [],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "schema_id": (str,),
                    "owner_id": (str,),
                    "document_id": (str,),
                    "owner_type": (int,),
                    "object_type": (int,),
                    "strict": (bool,),
                    "active_only": (bool,),
                    "offset": (int,),
                    "limit": (int,),
                },
                "attribute_map": {
                    "schema_id": "schemaId",
                    "owner_id": "ownerId",
                    "document_id": "documentId",
                    "owner_type": "ownerType",
                    "object_type": "objectType",
                    "strict": "strict",
                    "active_only": "activeOnly",
                    "offset": "offset",
                    "limit": "limit",
                },
                "location_map": {
                    "schema_id": "query",
                    "owner_id": "query",
                    "document_id": "query",
                    "owner_type": "query",
                    "object_type": "query",
                    "strict": "query",
                    "active_only": "query",
                    "offset": "query",
                    "limit": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__get_properties,
        )

        def __get_property_info(self, pid, **kwargs):
            """get_property_info  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_property_info(pid, async_req=True)
            >>> result = thread.get()

            Args:
                pid (str):

            Keyword Args:
                document_id (str): [optional]
                schema_id (str): [optional]
                owner_id (str): [optional]
                owner_type (int): [optional] if omitted the server will use the default value of 1
                object_type (int): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_metadata_property_info.BTMetadataPropertyInfo
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["pid"] = pid
            return self.call_with_http_info(**kwargs)

        self.get_property_info = Endpoint(
            settings={
                "response_type": (bt_metadata_property_info.BTMetadataPropertyInfo,),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/metadataschema/propertyinfo/{pid}",
                "operation_id": "get_property_info",
                "http_method": "GET",
                "servers": [],
            },
            params_map={
                "all": [
                    "pid",
                    "document_id",
                    "schema_id",
                    "owner_id",
                    "owner_type",
                    "object_type",
                ],
                "required": ["pid",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "pid": (str,),
                    "document_id": (str,),
                    "schema_id": (str,),
                    "owner_id": (str,),
                    "owner_type": (int,),
                    "object_type": (int,),
                },
                "attribute_map": {
                    "pid": "pid",
                    "document_id": "documentId",
                    "schema_id": "schemaId",
                    "owner_id": "ownerId",
                    "owner_type": "ownerType",
                    "object_type": "objectType",
                },
                "location_map": {
                    "pid": "path",
                    "document_id": "query",
                    "schema_id": "query",
                    "owner_id": "query",
                    "owner_type": "query",
                    "object_type": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__get_property_info,
        )

        def __get_schema(self, sid, **kwargs):
            """get_schema  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_schema(sid, async_req=True)
            >>> result = thread.get()

            Args:
                sid (str):

            Keyword Args:
                document_id (str): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_metadata_schema_info.BTMetadataSchemaInfo
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["sid"] = sid
            return self.call_with_http_info(**kwargs)

        self.get_schema = Endpoint(
            settings={
                "response_type": (bt_metadata_schema_info.BTMetadataSchemaInfo,),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/metadataschema/{sid}",
                "operation_id": "get_schema",
                "http_method": "GET",
                "servers": [],
            },
            params_map={
                "all": ["sid", "document_id",],
                "required": ["sid",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {"sid": (str,), "document_id": (str,),},
                "attribute_map": {"sid": "sid", "document_id": "documentId",},
                "location_map": {"sid": "path", "document_id": "query",},
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__get_schema,
        )

        def __get_sketch_bounding_boxes(self, did, wvm, wvmid, eid, sid, **kwargs):
            """get_sketch_bounding_boxes  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_sketch_bounding_boxes(did, wvm, wvmid, eid, sid, async_req=True)
            >>> result = thread.get()

            Args:
                did (str):
                wvm (str):
                wvmid (str):
                eid (str):
                sid (str):

            Keyword Args:
                configuration (str): [optional]
                link_document_id (str): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_bounding_box_info.BTBoundingBoxInfo
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["did"] = did
            kwargs["wvm"] = wvm
            kwargs["wvmid"] = wvmid
            kwargs["eid"] = eid
            kwargs["sid"] = sid
            return self.call_with_http_info(**kwargs)

        self.get_sketch_bounding_boxes = Endpoint(
            settings={
                "response_type": (bt_bounding_box_info.BTBoundingBoxInfo,),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/partstudios/d/{did}/{wvm}/{wvmid}/e/{eid}/sketches/{sid}/boundingboxes",
                "operation_id": "get_sketch_bounding_boxes",
                "http_method": "GET",
                "servers": [],
            },
            params_map={
                "all": [
                    "did",
                    "wvm",
                    "wvmid",
                    "eid",
                    "sid",
                    "configuration",
                    "link_document_id",
                ],
                "required": ["did", "wvm", "wvmid", "eid", "sid",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "did": (str,),
                    "wvm": (str,),
                    "wvmid": (str,),
                    "eid": (str,),
                    "sid": (str,),
                    "configuration": (str,),
                    "link_document_id": (str,),
                },
                "attribute_map": {
                    "did": "did",
                    "wvm": "wvm",
                    "wvmid": "wvmid",
                    "eid": "eid",
                    "sid": "sid",
                    "configuration": "configuration",
                    "link_document_id": "linkDocumentId",
                },
                "location_map": {
                    "did": "path",
                    "wvm": "path",
                    "wvmid": "path",
                    "eid": "path",
                    "sid": "path",
                    "configuration": "query",
                    "link_document_id": "query",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__get_sketch_bounding_boxes,
        )

        def __get_sketch_info(self, did, wvm, wvmid, eid, **kwargs):
            """get_sketch_info  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_sketch_info(did, wvm, wvmid, eid, async_req=True)
            >>> result = thread.get()

            Args:
                did (str):
                wvm (str):
                wvmid (str):
                eid (str):

            Keyword Args:
                configuration (str): [optional]
                sketch_id ([str]): [optional]
                output3_d (bool): [optional] if omitted the server will use the default value of False
                curve_points (bool): [optional] if omitted the server will use the default value of False
                include_geometry (bool): [optional] if omitted the server will use the default value of True
                link_document_id (str): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["did"] = did
            kwargs["wvm"] = wvm
            kwargs["wvmid"] = wvmid
            kwargs["eid"] = eid
            return self.call_with_http_info(**kwargs)

        self.get_sketch_info = Endpoint(
            settings={
                "response_type": None,
                "auth": ["OAuth2"],
                "endpoint_path": "/api/partstudios/d/{did}/{wvm}/{wvmid}/e/{eid}/sketches",
                "operation_id": "get_sketch_info",
                "http_method": "GET",
                "servers": [],
            },
            params_map={
                "all": [
                    "did",
                    "wvm",
                    "wvmid",
                    "eid",
                    "configuration",
                    "sketch_id",
                    "output3_d",
                    "curve_points",
                    "include_geometry",
                    "link_document_id",
                ],
                "required": ["did", "wvm", "wvmid", "eid",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "did": (str,),
                    "wvm": (str,),
                    "wvmid": (str,),
                    "eid": (str,),
                    "configuration": (str,),
                    "sketch_id": ([str],),
                    "output3_d": (bool,),
                    "curve_points": (bool,),
                    "include_geometry": (bool,),
                    "link_document_id": (str,),
                },
                "attribute_map": {
                    "did": "did",
                    "wvm": "wvm",
                    "wvmid": "wvmid",
                    "eid": "eid",
                    "configuration": "configuration",
                    "sketch_id": "sketchId",
                    "output3_d": "output3D",
                    "curve_points": "curvePoints",
                    "include_geometry": "includeGeometry",
                    "link_document_id": "linkDocumentId",
                },
                "location_map": {
                    "did": "path",
                    "wvm": "path",
                    "wvmid": "path",
                    "eid": "path",
                    "configuration": "query",
                    "sketch_id": "query",
                    "output3_d": "query",
                    "curve_points": "query",
                    "include_geometry": "query",
                    "link_document_id": "query",
                },
                "collection_format_map": {"sketch_id": "multi",},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__get_sketch_info,
        )

        def __get_tessellated_entities(self, did, wvm, wvmid, eid, sid, **kwargs):
            """get_tessellated_entities  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_tessellated_entities(did, wvm, wvmid, eid, sid, async_req=True)
            >>> result = thread.get()

            Args:
                did (str):
                wvm (str):
                wvmid (str):
                eid (str):
                sid (str):

            Keyword Args:
                configuration (str): [optional]
                entity_id ([str]): [optional]
                angle_tolerance (float): [optional]
                chord_tolerance (float): [optional]
                link_document_id (str): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["did"] = did
            kwargs["wvm"] = wvm
            kwargs["wvmid"] = wvmid
            kwargs["eid"] = eid
            kwargs["sid"] = sid
            return self.call_with_http_info(**kwargs)

        self.get_tessellated_entities = Endpoint(
            settings={
                "response_type": None,
                "auth": ["OAuth2"],
                "endpoint_path": "/api/partstudios/d/{did}/{wvm}/{wvmid}/e/{eid}/sketches/{sid}/tessellatedentities",
                "operation_id": "get_tessellated_entities",
                "http_method": "GET",
                "servers": [],
            },
            params_map={
                "all": [
                    "did",
                    "wvm",
                    "wvmid",
                    "eid",
                    "sid",
                    "configuration",
                    "entity_id",
                    "angle_tolerance",
                    "chord_tolerance",
                    "link_document_id",
                ],
                "required": ["did", "wvm", "wvmid", "eid", "sid",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "did": (str,),
                    "wvm": (str,),
                    "wvmid": (str,),
                    "eid": (str,),
                    "sid": (str,),
                    "configuration": (str,),
                    "entity_id": ([str],),
                    "angle_tolerance": (float,),
                    "chord_tolerance": (float,),
                    "link_document_id": (str,),
                },
                "attribute_map": {
                    "did": "did",
                    "wvm": "wvm",
                    "wvmid": "wvmid",
                    "eid": "eid",
                    "sid": "sid",
                    "configuration": "configuration",
                    "entity_id": "entityId",
                    "angle_tolerance": "angleTolerance",
                    "chord_tolerance": "chordTolerance",
                    "link_document_id": "linkDocumentId",
                },
                "location_map": {
                    "did": "path",
                    "wvm": "path",
                    "wvmid": "path",
                    "eid": "path",
                    "sid": "path",
                    "configuration": "query",
                    "entity_id": "query",
                    "angle_tolerance": "query",
                    "chord_tolerance": "query",
                    "link_document_id": "query",
                },
                "collection_format_map": {"entity_id": "multi",},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__get_tessellated_entities,
        )

        def __get_workflowable_test_object(self, oid, **kwargs):
            """get_workflowable_test_object  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_workflowable_test_object(oid, async_req=True)
            >>> result = thread.get()

            Args:
                oid (str):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_workflowable_test_object_info.BTWorkflowableTestObjectInfo
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["oid"] = oid
            return self.call_with_http_info(**kwargs)

        self.get_workflowable_test_object = Endpoint(
            settings={
                "response_type": (
                    bt_workflowable_test_object_info.BTWorkflowableTestObjectInfo,
                ),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/workflowabletestobject/{oid}",
                "operation_id": "get_workflowable_test_object",
                "http_method": "GET",
                "servers": [],
            },
            params_map={
                "all": ["oid",],
                "required": ["oid",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {"oid": (str,),},
                "attribute_map": {"oid": "oid",},
                "location_map": {"oid": "path",},
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__get_workflowable_test_object,
        )

        def __post_associative_data(self, did, eid, wvm, wvmid, body, **kwargs):
            """post_associative_data  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.post_associative_data(did, eid, wvm, wvmid, body, async_req=True)
            >>> result = thread.get()

            Args:
                did (str):
                eid (str):
                wvm (str):
                wvmid (str):
                body (str):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_app_associative_data_info_array.BTAppAssociativeDataInfoArray
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["did"] = did
            kwargs["eid"] = eid
            kwargs["wvm"] = wvm
            kwargs["wvmid"] = wvmid
            kwargs["body"] = body
            return self.call_with_http_info(**kwargs)

        self.post_associative_data = Endpoint(
            settings={
                "response_type": (
                    bt_app_associative_data_info_array.BTAppAssociativeDataInfoArray,
                ),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/appelements/d/{did}/{wvm}/{wvmid}/e/{eid}/associativedata",
                "operation_id": "post_associative_data",
                "http_method": "POST",
                "servers": [],
            },
            params_map={
                "all": ["did", "eid", "wvm", "wvmid", "body",],
                "required": ["did", "eid", "wvm", "wvmid", "body",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "did": (str,),
                    "eid": (str,),
                    "wvm": (str,),
                    "wvmid": (str,),
                    "body": (str,),
                },
                "attribute_map": {
                    "did": "did",
                    "eid": "eid",
                    "wvm": "wvm",
                    "wvmid": "wvmid",
                },
                "location_map": {
                    "did": "path",
                    "eid": "path",
                    "wvm": "path",
                    "wvmid": "path",
                    "body": "body",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": ["application/json;charset=UTF-8; qs=0.09"],
            },
            api_client=api_client,
            callable=__post_associative_data,
        )

        def __transition_workflowable_test_object(self, oid, transition, **kwargs):
            """transition_workflowable_test_object  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.transition_workflowable_test_object(oid, transition, async_req=True)
            >>> result = thread.get()

            Args:
                oid (str):
                transition (str):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_workflowable_test_object_info.BTWorkflowableTestObjectInfo
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["oid"] = oid
            kwargs["transition"] = transition
            return self.call_with_http_info(**kwargs)

        self.transition_workflowable_test_object = Endpoint(
            settings={
                "response_type": (
                    bt_workflowable_test_object_info.BTWorkflowableTestObjectInfo,
                ),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/workflowabletestobject/{oid}/{transition}",
                "operation_id": "transition_workflowable_test_object",
                "http_method": "POST",
                "servers": [],
            },
            params_map={
                "all": ["oid", "transition",],
                "required": ["oid", "transition",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {"oid": (str,), "transition": (str,),},
                "attribute_map": {"oid": "oid", "transition": "transition",},
                "location_map": {"oid": "path", "transition": "path",},
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": [],
            },
            api_client=api_client,
            callable=__transition_workflowable_test_object,
        )

        def __update_workflowable_test_object(
            self, oid, bt_update_workflowable_test_object_params, **kwargs
        ):
            """update_workflowable_test_object  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.update_workflowable_test_object(oid, bt_update_workflowable_test_object_params, async_req=True)
            >>> result = thread.get()

            Args:
                oid (str):
                bt_update_workflowable_test_object_params (bt_update_workflowable_test_object_params.BTUpdateWorkflowableTestObjectParams):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bt_workflowable_test_object_info.BTWorkflowableTestObjectInfo
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs["async_req"] = kwargs.get("async_req", False)
            kwargs["_return_http_data_only"] = kwargs.get(
                "_return_http_data_only", True
            )
            kwargs["_preload_content"] = kwargs.get("_preload_content", True)
            kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
            kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
            kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
            kwargs["_host_index"] = kwargs.get("_host_index", 0)
            kwargs["oid"] = oid
            kwargs[
                "bt_update_workflowable_test_object_params"
            ] = bt_update_workflowable_test_object_params
            return self.call_with_http_info(**kwargs)

        self.update_workflowable_test_object = Endpoint(
            settings={
                "response_type": (
                    bt_workflowable_test_object_info.BTWorkflowableTestObjectInfo,
                ),
                "auth": ["OAuth2"],
                "endpoint_path": "/api/workflowabletestobject/{oid}",
                "operation_id": "update_workflowable_test_object",
                "http_method": "POST",
                "servers": [],
            },
            params_map={
                "all": ["oid", "bt_update_workflowable_test_object_params",],
                "required": ["oid", "bt_update_workflowable_test_object_params",],
                "nullable": [],
                "enum": [],
                "validation": [],
            },
            root_map={
                "validations": {},
                "allowed_values": {},
                "openapi_types": {
                    "oid": (str,),
                    "bt_update_workflowable_test_object_params": (
                        bt_update_workflowable_test_object_params.BTUpdateWorkflowableTestObjectParams,
                    ),
                },
                "attribute_map": {"oid": "oid",},
                "location_map": {
                    "oid": "path",
                    "bt_update_workflowable_test_object_params": "body",
                },
                "collection_format_map": {},
            },
            headers_map={
                "accept": ["application/vnd.onshape.v1+json;charset=UTF-8;qs=0.1"],
                "content_type": ["application/json;charset=UTF-8; qs=0.09"],
            },
            api_client=api_client,
            callable=__update_workflowable_test_object,
        )


class Endpoint(object):
    def __init__(
        self,
        settings=None,
        params_map=None,
        root_map=None,
        headers_map=None,
        api_client=None,
        callable=None,
    ):
        """Creates an endpoint

        Args:
            settings (dict): see below key value pairs
                'response_type' (tuple/None): response type
                'auth' (list): a list of auth type keys
                'endpoint_path' (str): the endpoint path
                'operation_id' (str): endpoint string identifier
                'http_method' (str): POST/PUT/PATCH/GET etc
                'servers' (list): list of str servers that this endpoint is at
            params_map (dict): see below key value pairs
                'all' (list): list of str endpoint parameter names
                'required' (list): list of required parameter names
                'nullable' (list): list of nullable parameter names
                'enum' (list): list of parameters with enum values
                'validation' (list): list of parameters with validations
            root_map
                'validations' (dict): the dict mapping endpoint parameter tuple
                    paths to their validation dictionaries
                'allowed_values' (dict): the dict mapping endpoint parameter
                    tuple paths to their allowed_values (enum) dictionaries
                'openapi_types' (dict): param_name to openapi type
                'attribute_map' (dict): param_name to camelCase name
                'location_map' (dict): param_name to  'body', 'file', 'form',
                    'header', 'path', 'query'
                collection_format_map (dict): param_name to `csv` etc.
            headers_map (dict): see below key value pairs
                'accept' (list): list of Accept header strings
                'content_type' (list): list of Content-Type header strings
            api_client (ApiClient) api client instance
            callable (function): the function which is invoked when the
                Endpoint is called
        """
        self.settings = settings
        self.params_map = params_map
        self.params_map["all"].extend(
            [
                "async_req",
                "_host_index",
                "_preload_content",
                "_request_timeout",
                "_return_http_data_only",
                "_check_input_type",
                "_check_return_type",
            ]
        )
        self.params_map["nullable"].extend(["_request_timeout"])
        self.validations = root_map["validations"]
        self.allowed_values = root_map["allowed_values"]
        self.openapi_types = root_map["openapi_types"]
        extra_types = {
            "async_req": (bool,),
            "_host_index": (int,),
            "_preload_content": (bool,),
            "_request_timeout": (none_type, int, (int,), [int]),
            "_return_http_data_only": (bool,),
            "_check_input_type": (bool,),
            "_check_return_type": (bool,),
        }
        self.openapi_types.update(extra_types)
        self.attribute_map = root_map["attribute_map"]
        self.location_map = root_map["location_map"]
        self.collection_format_map = root_map["collection_format_map"]
        self.headers_map = headers_map
        self.api_client = api_client
        self.callable = callable

    def __validate_inputs(self, kwargs):
        for param in self.params_map["enum"]:
            if param in kwargs:
                check_allowed_values(self.allowed_values, (param,), kwargs[param])

        for param in self.params_map["validation"]:
            if param in kwargs:
                check_validations(self.validations, (param,), kwargs[param])

        if kwargs["_check_input_type"] is False:
            return

        for key, value in six.iteritems(kwargs):
            fixed_val = validate_and_convert_types(
                value,
                self.openapi_types[key],
                [key],
                False,
                kwargs["_check_input_type"],
                configuration=self.api_client.configuration,
            )
            kwargs[key] = fixed_val

    def __gather_params(self, kwargs):
        params = {
            "body": None,
            "collection_format": {},
            "file": {},
            "form": [],
            "header": {},
            "path": {},
            "query": [],
        }

        for param_name, param_value in six.iteritems(kwargs):
            param_location = self.location_map.get(param_name)
            if param_location is None:
                continue
            if param_location:
                if param_location == "body":
                    params["body"] = param_value
                    continue
                base_name = self.attribute_map[param_name]
                if param_location == "form" and self.openapi_types[param_name] == (
                    file_type,
                ):
                    params["file"][param_name] = [param_value]
                elif param_location == "form" and self.openapi_types[param_name] == (
                    [file_type],
                ):
                    # param_value is already a list
                    params["file"][param_name] = param_value
                elif param_location in {"form", "query"}:
                    param_value_full = (base_name, param_value)
                    params[param_location].append(param_value_full)
                if param_location not in {"form", "query"}:
                    params[param_location][base_name] = param_value
                collection_format = self.collection_format_map.get(param_name)
                if collection_format:
                    params["collection_format"][base_name] = collection_format

        return params

    def __call__(self, *args, **kwargs):
        """ This method is invoked when endpoints are called
        Example:
        pet_api = PetApi()
        pet_api.add_pet  # this is an instance of the class Endpoint
        pet_api.add_pet()  # this invokes pet_api.add_pet.__call__()
        which then invokes the callable functions stored in that endpoint at
        pet_api.add_pet.callable or self.callable in this class
        """
        return self.callable(self, *args, **kwargs)

    def call_with_http_info(self, **kwargs):

        try:
            _host = self.settings["servers"][kwargs["_host_index"]]
        except IndexError:
            if self.settings["servers"]:
                raise ApiValueError(
                    "Invalid host index. Must be 0 <= index < %s"
                    % len(self.settings["servers"])
                )
            _host = None

        for key, value in six.iteritems(kwargs):
            if key not in self.params_map["all"]:
                raise ApiTypeError(
                    "Got an unexpected parameter '%s'"
                    " to method `%s`" % (key, self.settings["operation_id"])
                )
            # only throw this nullable ApiValueError if _check_input_type
            # is False, if _check_input_type==True we catch this case
            # in self.__validate_inputs
            if (
                key not in self.params_map["nullable"]
                and value is None
                and kwargs["_check_input_type"] is False
            ):
                raise ApiValueError(
                    "Value may not be None for non-nullable parameter `%s`"
                    " when calling `%s`" % (key, self.settings["operation_id"])
                )

        for key in self.params_map["required"]:
            if key not in kwargs.keys():
                raise ApiValueError(
                    "Missing the required parameter `%s` when calling "
                    "`%s`" % (key, self.settings["operation_id"])
                )

        self.__validate_inputs(kwargs)

        params = self.__gather_params(kwargs)

        accept_headers_list = self.headers_map["accept"]
        if accept_headers_list:
            params["header"]["Accept"] = self.api_client.select_header_accept(
                accept_headers_list
            )

        content_type_headers_list = self.headers_map["content_type"]
        if content_type_headers_list:
            header_list = self.api_client.select_header_content_type(
                content_type_headers_list
            )
            params["header"]["Content-Type"] = header_list

        return self.api_client.call_api(
            self.settings["endpoint_path"],
            self.settings["http_method"],
            params["path"],
            params["query"],
            params["header"],
            body=params["body"],
            post_params=params["form"],
            files=params["file"],
            response_type=self.settings["response_type"],
            auth_settings=self.settings["auth"],
            async_req=kwargs["async_req"],
            _check_type=kwargs["_check_return_type"],
            _return_http_data_only=kwargs["_return_http_data_only"],
            _preload_content=kwargs["_preload_content"],
            _request_timeout=kwargs["_request_timeout"],
            _host=_host,
            collection_formats=params["collection_format"],
        )



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    The version of the OpenAPI document: 1.113
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import
import re  # noqa: F401
import sys  # noqa: F401

import six  # noqa: F401
import nulltype  # noqa: F401

from onshape_client.oas.model_utils import (  # noqa: F401
    ModelComposed,
    ModelNormal,
    ModelSimple,
    date,
    datetime,
    file_type,
    int,
    none_type,
    str,
    validate_get_composed_info,
)

try:
    from onshape_client.oas.models import bt_base_info
except ImportError:
    bt_base_info = sys.modules["onshape_client.oas.models.bt_base_info"]
try:
    from onshape_client.oas.models import bt_document_info
except ImportError:
    bt_document_info = sys.modules["onshape_client.oas.models.bt_document_info"]
try:
    from onshape_client.oas.models import bt_document_label_info
except ImportError:
    bt_document_label_info = sys.modules[
        "onshape_client.oas.models.bt_document_label_info"
    ]
try:
    from onshape_client.oas.models import bt_document_summary_info_all_of
except ImportError:
    bt_document_summary_info_all_of = sys.modules[
        "onshape_client.oas.models.bt_document_summary_info_all_of"
    ]
try:
    from onshape_client.oas.models import bt_document_summary_search_info
except ImportError:
    bt_document_summary_search_info = sys.modules[
        "onshape_client.oas.models.bt_document_summary_search_info"
    ]
try:
    from onshape_client.oas.models import bt_global_tree_node_info
except ImportError:
    bt_global_tree_node_info = sys.modules[
        "onshape_client.oas.models.bt_global_tree_node_info"
    ]
try:
    from onshape_client.oas.models import bt_owner_info
except ImportError:
    bt_owner_info = sys.modules["onshape_client.oas.models.bt_owner_info"]
try:
    from onshape_client.oas.models import bt_thumbnail_info
except ImportError:
    bt_thumbnail_info = sys.modules["onshape_client.oas.models.bt_thumbnail_info"]
try:
    from onshape_client.oas.models import bt_user_basic_summary_info
except ImportError:
    bt_user_basic_summary_info = sys.modules[
        "onshape_client.oas.models.bt_user_basic_summary_info"
    ]
try:
    from onshape_client.oas.models import bt_workspace_info
except ImportError:
    bt_workspace_info = sys.modules["onshape_client.oas.models.bt_workspace_info"]


class BTDocumentSummaryInfo(ModelComposed):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ("permission",): {
            "NOACCESS": "NOACCESS",
            "ANONYMOUS_ACCESS": "ANONYMOUS_ACCESS",
            "READ": "READ",
            "READ_COPY_EXPORT": "READ_COPY_EXPORT",
            "COMMENT": "COMMENT",
            "WRITE": "WRITE",
            "RESHARE": "RESHARE",
            "FULL": "FULL",
            "OWNER": "OWNER",
        },
    }

    validations = {}

    additional_properties_type = None

    @staticmethod
    def openapi_types():
        """
        This must be a class method so a model may have properties that are
        of type self, this ensures that we don't create a cyclic import

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        return {
            "json_type": (str,),  # noqa: E501
            "anonymous_access_allowed": (bool,),  # noqa: E501
            "anonymous_allows_export": (bool,),  # noqa: E501
            "can_unshare": (bool,),  # noqa: E501
            "created_with_education_plan": (bool,),  # noqa: E501
            "default_element_id": (str,),  # noqa: E501
            "default_workspace": (bt_workspace_info.BTWorkspaceInfo,),  # noqa: E501
            "document_labels": (
                [bt_document_label_info.BTDocumentLabelInfo],
            ),  # noqa: E501
            "has_release_revisionable_objects": (bool,),  # noqa: E501
            "has_relevant_insertables": (bool,),  # noqa: E501
            "is_orphaned": (bool,),  # noqa: E501
            "is_using_managed_workflow": (bool,),  # noqa: E501
            "liked_by_current_user": (bool,),  # noqa: E501
            "likes": (int,),  # noqa: E501
            "not_revision_managed": (bool,),  # noqa: E501
            "number_of_times_copied": (int,),  # noqa: E501
            "number_of_times_referenced": (int,),  # noqa: E501
            "parent_id": (str,),  # noqa: E501
            "permission": (str,),  # noqa: E501
            "permission_set": ([str],),  # noqa: E501
            "public": (bool,),  # noqa: E501
            "recent_version": (bt_base_info.BTBaseInfo,),  # noqa: E501
            "support_team_user_and_shared": (bool,),  # noqa: E501
            "tags": ([str],),  # noqa: E501
            "thumbnail": (bt_thumbnail_info.BTThumbnailInfo,),  # noqa: E501
            "total_workspaces_scheduled_for_update": (int,),  # noqa: E501
            "total_workspaces_updating": (int,),  # noqa: E501
            "trash": (bool,),  # noqa: E501
            "trashed_at": (datetime,),  # noqa: E501
            "user_account_limits_breached": (bool,),  # noqa: E501
            "can_move": (bool,),  # noqa: E501
            "created_at": (datetime,),  # noqa: E501
            "created_by": (
                bt_user_basic_summary_info.BTUserBasicSummaryInfo,
            ),  # noqa: E501
            "description": (str,),  # noqa: E501
            "href": (str,),  # noqa: E501
            "id": (str,),  # noqa: E501
            "is_container": (bool,),  # noqa: E501
            "is_enterprise_owned": (bool,),  # noqa: E501
            "is_mutable": (bool,),  # noqa: E501
            "modified_at": (datetime,),  # noqa: E501
            "modified_by": (
                bt_user_basic_summary_info.BTUserBasicSummaryInfo,
            ),  # noqa: E501
            "name": (str,),  # noqa: E501
            "owner": (bt_owner_info.BTOwnerInfo,),  # noqa: E501
            "project_id": (str,),  # noqa: E501
            "resource_type": (str,),  # noqa: E501
            "tree_href": (str,),  # noqa: E501
            "view_ref": (str,),  # noqa: E501
        }

    @staticmethod
    def discriminator():
        return {
            "json_type": {
                "BTDocumentSummarySearchInfo": bt_document_summary_search_info.BTDocumentSummarySearchInfo,
            },
        }

    attribute_map = {
        "json_type": "jsonType",  # noqa: E501
        "anonymous_access_allowed": "anonymousAccessAllowed",  # noqa: E501
        "anonymous_allows_export": "anonymousAllowsExport",  # noqa: E501
        "can_unshare": "canUnshare",  # noqa: E501
        "created_with_education_plan": "createdWithEducationPlan",  # noqa: E501
        "default_element_id": "defaultElementId",  # noqa: E501
        "default_workspace": "defaultWorkspace",  # noqa: E501
        "document_labels": "documentLabels",  # noqa: E501
        "has_release_revisionable_objects": "hasReleaseRevisionableObjects",  # noqa: E501
        "has_relevant_insertables": "hasRelevantInsertables",  # noqa: E501
        "is_orphaned": "isOrphaned",  # noqa: E501
        "is_using_managed_workflow": "isUsingManagedWorkflow",  # noqa: E501
        "liked_by_current_user": "likedByCurrentUser",  # noqa: E501
        "likes": "likes",  # noqa: E501
        "not_revision_managed": "notRevisionManaged",  # noqa: E501
        "number_of_times_copied": "numberOfTimesCopied",  # noqa: E501
        "number_of_times_referenced": "numberOfTimesReferenced",  # noqa: E501
        "parent_id": "parentId",  # noqa: E501
        "permission": "permission",  # noqa: E501
        "permission_set": "permissionSet",  # noqa: E501
        "public": "public",  # noqa: E501
        "recent_version": "recentVersion",  # noqa: E501
        "support_team_user_and_shared": "supportTeamUserAndShared",  # noqa: E501
        "tags": "tags",  # noqa: E501
        "thumbnail": "thumbnail",  # noqa: E501
        "total_workspaces_scheduled_for_update": "totalWorkspacesScheduledForUpdate",  # noqa: E501
        "total_workspaces_updating": "totalWorkspacesUpdating",  # noqa: E501
        "trash": "trash",  # noqa: E501
        "trashed_at": "trashedAt",  # noqa: E501
        "user_account_limits_breached": "userAccountLimitsBreached",  # noqa: E501
        "can_move": "canMove",  # noqa: E501
        "created_at": "createdAt",  # noqa: E501
        "created_by": "createdBy",  # noqa: E501
        "description": "description",  # noqa: E501
        "href": "href",  # noqa: E501
        "id": "id",  # noqa: E501
        "is_container": "isContainer",  # noqa: E501
        "is_enterprise_owned": "isEnterpriseOwned",  # noqa: E501
        "is_mutable": "isMutable",  # noqa: E501
        "modified_at": "modifiedAt",  # noqa: E501
        "modified_by": "modifiedBy",  # noqa: E501
        "name": "name",  # noqa: E501
        "owner": "owner",  # noqa: E501
        "project_id": "projectId",  # noqa: E501
        "resource_type": "resourceType",  # noqa: E501
        "tree_href": "treeHref",  # noqa: E501
        "view_ref": "viewRef",  # noqa: E501
    }

    required_properties = set(
        [
            "_data_store",
            "_check_type",
            "_from_server",
            "_path_to_item",
            "_configuration",
            "_composed_instances",
            "_var_name_to_model_instances",
            "_additional_properties_model_instances",
        ]
    )

    def __init__(
        self,
        _check_type=True,
        _from_server=False,
        _path_to_item=(),
        _configuration=None,
        **kwargs
    ):  # noqa: E501
        """bt_document_summary_info.BTDocumentSummaryInfo - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _from_server (bool): True if the data is from the server
                                False if the data is from the client (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            json_type (str): [optional]  # noqa: E501
            anonymous_access_allowed (bool): [optional]  # noqa: E501
            anonymous_allows_export (bool): [optional]  # noqa: E501
            can_unshare (bool): [optional]  # noqa: E501
            created_with_education_plan (bool): [optional]  # noqa: E501
            default_element_id (str): [optional]  # noqa: E501
            default_workspace (bt_workspace_info.BTWorkspaceInfo): [optional]  # noqa: E501
            document_labels ([bt_document_label_info.BTDocumentLabelInfo]): [optional]  # noqa: E501
            has_release_revisionable_objects (bool): [optional]  # noqa: E501
            has_relevant_insertables (bool): [optional]  # noqa: E501
            is_orphaned (bool): [optional]  # noqa: E501
            is_using_managed_workflow (bool): [optional]  # noqa: E501
            liked_by_current_user (bool): [optional]  # noqa: E501
            likes (int): [optional]  # noqa: E501
            not_revision_managed (bool): [optional]  # noqa: E501
            number_of_times_copied (int): [optional]  # noqa: E501
            number_of_times_referenced (int): [optional]  # noqa: E501
            parent_id (str): [optional]  # noqa: E501
            permission (str): [optional]  # noqa: E501
            permission_set ([str]): [optional]  # noqa: E501
            public (bool): [optional]  # noqa: E501
            recent_version (bt_base_info.BTBaseInfo): [optional]  # noqa: E501
            support_team_user_and_shared (bool): [optional]  # noqa: E501
            tags ([str]): [optional]  # noqa: E501
            thumbnail (bt_thumbnail_info.BTThumbnailInfo): [optional]  # noqa: E501
            total_workspaces_scheduled_for_update (int): [optional]  # noqa: E501
            total_workspaces_updating (int): [optional]  # noqa: E501
            trash (bool): [optional]  # noqa: E501
            trashed_at (datetime): [optional]  # noqa: E501
            user_account_limits_breached (bool): [optional]  # noqa: E501
            can_move (bool): [optional]  # noqa: E501
            created_at (datetime): [optional]  # noqa: E501
            created_by (bt_user_basic_summary_info.BTUserBasicSummaryInfo): [optional]  # noqa: E501
            description (str): [optional]  # noqa: E501
            href (str): [optional]  # noqa: E501
            id (str): [optional]  # noqa: E501
            is_container (bool): [optional]  # noqa: E501
            is_enterprise_owned (bool): [optional]  # noqa: E501
            is_mutable (bool): [optional]  # noqa: E501
            modified_at (datetime): [optional]  # noqa: E501
            modified_by (bt_user_basic_summary_info.BTUserBasicSummaryInfo): [optional]  # noqa: E501
            name (str): [optional]  # noqa: E501
            owner (bt_owner_info.BTOwnerInfo): [optional]  # noqa: E501
            project_id (str): [optional]  # noqa: E501
            resource_type (str): [optional]  # noqa: E501
            tree_href (str): [optional]  # noqa: E501
            view_ref (str): [optional]  # noqa: E501
        """

        self._data_store = {}
        self._check_type = _check_type
        self._from_server = _from_server
        self._path_to_item = _path_to_item
        self._configuration = _configuration

        constant_args = {
            "_check_type": _check_type,
            "_path_to_item": _path_to_item,
            "_from_server": _from_server,
            "_configuration": _configuration,
        }
        required_args = {}
        # remove args whose value is Null because they are unset
        required_arg_names = list(required_args.keys())
        for required_arg_name in required_arg_names:
            if required_args[required_arg_name] is nulltype.Null:
                del required_args[required_arg_name]
        model_args = {}
        model_args.update(required_args)
        model_args.update(kwargs)
        composed_info = validate_get_composed_info(constant_args, model_args, self)
        self._composed_instances = composed_info[0]
        self._var_name_to_model_instances = composed_info[1]
        self._additional_properties_model_instances = composed_info[2]
        unused_args = composed_info[3]

        for var_name, var_value in required_args.items():
            setattr(self, var_name, var_value)
        for var_name, var_value in six.iteritems(kwargs):
            if (
                var_name in unused_args
                and self._configuration is not None
                and self._configuration.discard_unknown_keys
                and not self._additional_properties_model_instances
            ):
                # discard variable.
                continue
            setattr(self, var_name, var_value)

    @staticmethod
    def _composed_schemas():
        # we need this here to make our import statements work
        # we must store _composed_schemas in here so the code is only run
        # when we invoke this method. If we kept this at the class
        # level we would get an error beause the class level
        # code would be run when this module is imported, and these composed
        # classes don't exist yet because their module has not finished
        # loading
        return {
            "anyOf": [],
            "allOf": [
                bt_document_summary_info_all_of.BTDocumentSummaryInfoAllOf,
                bt_global_tree_node_info.BTGlobalTreeNodeInfo,
            ],
            "oneOf": [],
        }

    @classmethod
    def get_discriminator_class(cls, from_server, data):
        """Returns the child class specified by the discriminator"""
        discriminator = cls.discriminator()
        discr_propertyname_py = list(discriminator.keys())[0]
        discr_propertyname_js = cls.attribute_map[discr_propertyname_py]
        if from_server:
            class_name = data[discr_propertyname_js]
        else:
            class_name = data[discr_propertyname_py]
        class_name_to_discr_class = discriminator[discr_propertyname_py]
        return class_name_to_discr_class.get(class_name)



import json

import numpy as np
from onshape_client.client import get_client
from onshape_client.oas import BTAssemblyInstanceDefinitionParams, TransformGroup


class AssemblyDefinition:
    def __init__(self, assembly_definition, origin_element=None):
        self.assembly_definition = assembly_definition
        self.origin_element = origin_element

    def get_instance(self, occurrence_path):
        """Get the instance of this occurrence."""
        assembly_definition = self.assembly_definition
        if len(occurrence_path) == 0:
            return
        occ_path = occurrence_path.copy()
        # Consume assembly-pointing occurrence path elements
        instance = self.find_instance(
            occ_path.pop(0), assembly_definition["rootAssembly"]
        )
        while len(occ_path) > 0:
            sa = self.find_sub_assembly(instance)
            instance = self.find_instance(occ_path.pop(0), sa)
        return instance

    def make_sub_assembly_definition(self, sa_occurrence_path):
        """Get the AssemblyDefinition representing the subassembly."""
        sa = self.get_instance(sa_occurrence_path)
        get_client().assemblies_api.get_assembly_definition(
            sa["documentId"],
            "v",
            sa["documentVersion"],
            sa["elementId"],
            configuration=sa["configuration"],
            _preload_content=False,
        )
        return AssemblyDefinition(json.loads(result.data.decode("UTF-8")))

    @property
    def occurrences(self):
        return self.assembly_definition["rootAssembly"]["occurrences"]

    def get_occurrence_transforms(self, occurrence_path):
        """Get a list of all the occurrence transforms."""
        return [
            occ["transform"]
            for occ in self.occurrences
            if occ["path"] == occurrence_path
        ]

    def make_transform_group(self, occurrence_path):
        """Make the transform group for the particular occurrence."""
        instance = self.get_instance(occurrence_path)
        is_assembly = instance["type"] == "Assembly"
        transform = self.get_occurrence_transforms(occurrence_path)[0]

        # Special handling here to deal with Assembly instances to resolve to the correct transform.
        if is_assembly:
            sa = self.make_sub_assembly_definition(occurrence_path)
            occurrence_path.append(
                self.find_sub_assembly(instance)["instances"][0]["id"]
            )
            instance_transform_from_sa = sa.get_occurrence_transforms(
                [occurrence_path[-1]]
            )[0]
            instance_transform_from_root = self.get_occurrence_transforms(
                occurrence_path
            )[0]
            instance_transform_from_sa = np.reshape(
                np.array(instance_transform_from_sa), (4, 4)
            )
            instance_transform_from_root = np.reshape(
                np.array(instance_transform_from_root), (4, 4)
            )
            transform = np.matmul(
                instance_transform_from_root, np.linalg.inv(instance_transform_from_sa)
            )
            transform = transform.flatten().tolist()

        part_id = instance["partId"] if "partId" in instance else None
        if "documentVersion" in instance:
            doc_version = instance["documentVersion"]
        elif (
            "documentMicroversion" in instance
            and instance["documentMicroversion"]
            == self.assembly_definition["rootAssembly"]["documentMicroversion"]
        ):
            assert self.origin_element and self.origin_element.wvm == "v"
            doc_version = self.origin_element.wvmid
        else:
            raise UserWarning(
                "An assembly with linked documents needs to reference versions."
            )
        is_whole_part_studio = instance["type"] == "PartStudio"
        instance_definition = BTAssemblyInstanceDefinitionParams(
            document_id=instance["documentId"],
            version_id=doc_version,
            element_id=instance["elementId"],
            part_id=part_id,
            is_assembly=is_assembly,
            is_whole_part_studio=is_whole_part_studio,
            configuration=instance["configuration"],
        )
        return TransformGroup(transform=transform, instances=[instance_definition])

    def get_as_transform_groups(self):
        """Get a list of transform groups that define all the occurrences in this assembly."""
        result = []
        for occurrence in self.occurrences:
            result.append(self.make_transform_group(occurrence["path"]))
        return result

    def find_occurrence_paths(self, instance):
        """Get the occurrence path given an instance definition."""
        paths = []
        for occ in self.assembly_definition["rootAssembly"]["occurrences"]:
            potential_path = occ["path"]
            potential_instance = self.get_instance(potential_path)
            if self.is_equal_element_content(instance, potential_instance):
                paths.append(potential_path)
        return paths

    def find_instance(self, instance_id, sa):
        return next(
            instance for instance in sa["instances"] if instance["id"] == instance_id
        )

    def find_sub_assembly(self, instance):

        assembly_definition = self.assembly_definition
        for sa in assembly_definition["subAssemblies"]:
            if self.is_equal_element_content(sa, instance):
                return sa
        return None

    @staticmethod
    def compare_configurations(this, other):
        return set(this.split(";")) == set(other.split(";"))

    def is_structurally_equal_assembly(self, other):
        """ Returns true if the two assemblies have the same instances at the same location.
        :param other: AssemblyDefinition
        :return:
        """
        occurrences_to_be_matched = self.occurrences
        for occ in other.occurrences:
            transform_to_match = occ["transform"]
            instance_to_match = other.get_instance(occ["path"])
            for potential_occ in occurrences_to_be_matched:
                potential_transform = potential_occ["transform"]
                potential_instance = self.get_instance(potential_occ["path"])
                if (
                    AssemblyDefinition.is_equal_element_content(
                        potential_instance, instance_to_match
                    )
                    and transform_to_match == potential_transform
                ):
                    occurrences_to_be_matched.remove(potential_occ)
                    break
        return len(occurrences_to_be_matched) == 0

    @staticmethod
    def is_equal_element_content(this, other):
        return (
            this["documentMicroversion"] == other["documentMicroversion"]
            and this["elementId"] == other["elementId"]
            and AssemblyDefinition.compare_configurations(
                this["configuration"], other["configuration"]
            )
        )



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    The version of the OpenAPI document: 1.113
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import
import re  # noqa: F401
import sys  # noqa: F401

import six  # noqa: F401
import nulltype  # noqa: F401

from onshape_client.oas.model_utils import (  # noqa: F401
    ModelComposed,
    ModelNormal,
    ModelSimple,
    date,
    datetime,
    file_type,
    int,
    none_type,
    str,
    validate_get_composed_info,
)

try:
    from onshape_client.oas.models import bt_element_location_params
except ImportError:
    bt_element_location_params = sys.modules[
        "onshape_client.oas.models.bt_element_location_params"
    ]


class BTDrawingParams(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ("hidden_lines",): {
            "DRAFTING": "DRAFTING",
            "EXCLUDED": "EXCLUDED",
            "MARKED": "MARKED",
        },
        ("reference_type_enum",): {
            "UNKNOWN": "UNKNOWN",
            "PARTSTUDIO": "PARTSTUDIO",
            "ASSEMBLY": "ASSEMBLY",
            "PART": "PART",
            "FLATTENED_PART": "FLATTENED_PART",
            "COMPOSITE_PART": "COMPOSITE_PART",
            "MESH_PART": "MESH_PART",
            "SURFACE": "SURFACE",
            "SKETCH": "SKETCH",
            "CURVE": "CURVE",
        },
    }

    validations = {}

    additional_properties_type = None

    @staticmethod
    def openapi_types():
        """
        This must be a class method so a model may have properties that are
        of type self, this ensures that we don't create a cyclic import

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        return {
            "border": (bool,),  # noqa: E501
            "compute_intersection": (bool,),  # noqa: E501
            "decimal_separator": (str,),  # noqa: E501
            "document_id": (str,),  # noqa: E501
            "document_microversion_id": (str,),  # noqa: E501
            "drawing_name": (str,),  # noqa: E501
            "element_configuration": (str,),  # noqa: E501
            "element_id": (str,),  # noqa: E501
            "element_microversion_id": (str,),  # noqa: E501
            "external_document_id": (str,),  # noqa: E501
            "external_document_version_id": (str,),  # noqa: E501
            "hidden_lines": (str,),  # noqa: E501
            "include_surfaces": (bool,),  # noqa: E501
            "is_flattened_part": (bool,),  # noqa: E501
            "is_sketch_only": (bool,),  # noqa: E501
            "is_surface": (bool,),  # noqa: E501
            "language": (str,),  # noqa: E501
            "location": (
                bt_element_location_params.BTElementLocationParams,
            ),  # noqa: E501
            "model_type": (str,),  # noqa: E501
            "number_horizontal_zones": (int,),  # noqa: E501
            "number_vertical_zones": (int,),  # noqa: E501
            "part_id": (str,),  # noqa: E501
            "part_number": (str,),  # noqa: E501
            "part_query": (str,),  # noqa: E501
            "projection": (str,),  # noqa: E501
            "pure_sketch": (bool,),  # noqa: E501
            "quality_option": (str,),  # noqa: E501
            "reference_type": (int,),  # noqa: E501
            "reference_type_enum": (str,),  # noqa: E501
            "revision": (str,),  # noqa: E501
            "show_cut_geom_only": (bool,),  # noqa: E501
            "simplification_option": (str,),  # noqa: E501
            "simplification_threshold": (float,),  # noqa: E501
            "size": (str,),  # noqa: E501
            "sketch_ids": ([str],),  # noqa: E501
            "standard": (str,),  # noqa: E501
            "start_zones": (str,),  # noqa: E501
            "template_args": ([str],),  # noqa: E501
            "template_document_id": (str,),  # noqa: E501
            "template_element_id": (str,),  # noqa: E501
            "template_name": (str,),  # noqa: E501
            "template_version_id": (str,),  # noqa: E501
            "template_workspace_id": (str,),  # noqa: E501
            "titleblock": (bool,),  # noqa: E501
            "units": (str,),  # noqa: E501
            "views": (str,),  # noqa: E501
            "workspace_id": (str,),  # noqa: E501
        }

    @staticmethod
    def discriminator():
        return None

    attribute_map = {
        "border": "border",  # noqa: E501
        "compute_intersection": "computeIntersection",  # noqa: E501
        "decimal_separator": "decimalSeparator",  # noqa: E501
        "document_id": "documentId",  # noqa: E501
        "document_microversion_id": "documentMicroversionId",  # noqa: E501
        "drawing_name": "drawingName",  # noqa: E501
        "element_configuration": "elementConfiguration",  # noqa: E501
        "element_id": "elementId",  # noqa: E501
        "element_microversion_id": "elementMicroversionId",  # noqa: E501
        "external_document_id": "externalDocumentId",  # noqa: E501
        "external_document_version_id": "externalDocumentVersionId",  # noqa: E501
        "hidden_lines": "hiddenLines",  # noqa: E501
        "include_surfaces": "includeSurfaces",  # noqa: E501
        "is_flattened_part": "isFlattenedPart",  # noqa: E501
        "is_sketch_only": "isSketchOnly",  # noqa: E501
        "is_surface": "isSurface",  # noqa: E501
        "language": "language",  # noqa: E501
        "location": "location",  # noqa: E501
        "model_type": "modelType",  # noqa: E501
        "number_horizontal_zones": "numberHorizontalZones",  # noqa: E501
        "number_vertical_zones": "numberVerticalZones",  # noqa: E501
        "part_id": "partId",  # noqa: E501
        "part_number": "partNumber",  # noqa: E501
        "part_query": "partQuery",  # noqa: E501
        "projection": "projection",  # noqa: E501
        "pure_sketch": "pureSketch",  # noqa: E501
        "quality_option": "qualityOption",  # noqa: E501
        "reference_type": "referenceType",  # noqa: E501
        "reference_type_enum": "referenceTypeEnum",  # noqa: E501
        "revision": "revision",  # noqa: E501
        "show_cut_geom_only": "showCutGeomOnly",  # noqa: E501
        "simplification_option": "simplificationOption",  # noqa: E501
        "simplification_threshold": "simplificationThreshold",  # noqa: E501
        "size": "size",  # noqa: E501
        "sketch_ids": "sketchIds",  # noqa: E501
        "standard": "standard",  # noqa: E501
        "start_zones": "startZones",  # noqa: E501
        "template_args": "templateArgs",  # noqa: E501
        "template_document_id": "templateDocumentId",  # noqa: E501
        "template_element_id": "templateElementId",  # noqa: E501
        "template_name": "templateName",  # noqa: E501
        "template_version_id": "templateVersionId",  # noqa: E501
        "template_workspace_id": "templateWorkspaceId",  # noqa: E501
        "titleblock": "titleblock",  # noqa: E501
        "units": "units",  # noqa: E501
        "views": "views",  # noqa: E501
        "workspace_id": "workspaceId",  # noqa: E501
    }

    @staticmethod
    def _composed_schemas():
        return None

    required_properties = set(
        [
            "_data_store",
            "_check_type",
            "_from_server",
            "_path_to_item",
            "_configuration",
        ]
    )

    def __init__(
        self,
        _check_type=True,
        _from_server=False,
        _path_to_item=(),
        _configuration=None,
        **kwargs
    ):  # noqa: E501
        """bt_drawing_params.BTDrawingParams - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _from_server (bool): True if the data is from the server
                                False if the data is from the client (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            border (bool): [optional]  # noqa: E501
            compute_intersection (bool): [optional]  # noqa: E501
            decimal_separator (str): [optional]  # noqa: E501
            document_id (str): [optional]  # noqa: E501
            document_microversion_id (str): [optional]  # noqa: E501
            drawing_name (str): [optional]  # noqa: E501
            element_configuration (str): [optional]  # noqa: E501
            element_id (str): [optional]  # noqa: E501
            element_microversion_id (str): [optional]  # noqa: E501
            external_document_id (str): [optional]  # noqa: E501
            external_document_version_id (str): [optional]  # noqa: E501
            hidden_lines (str): [optional]  # noqa: E501
            include_surfaces (bool): [optional]  # noqa: E501
            is_flattened_part (bool): [optional]  # noqa: E501
            is_sketch_only (bool): [optional]  # noqa: E501
            is_surface (bool): [optional]  # noqa: E501
            language (str): [optional]  # noqa: E501
            location (bt_element_location_params.BTElementLocationParams): [optional]  # noqa: E501
            model_type (str): [optional]  # noqa: E501
            number_horizontal_zones (int): [optional]  # noqa: E501
            number_vertical_zones (int): [optional]  # noqa: E501
            part_id (str): [optional]  # noqa: E501
            part_number (str): [optional]  # noqa: E501
            part_query (str): [optional]  # noqa: E501
            projection (str): [optional]  # noqa: E501
            pure_sketch (bool): [optional]  # noqa: E501
            quality_option (str): [optional]  # noqa: E501
            reference_type (int): [optional]  # noqa: E501
            reference_type_enum (str): [optional]  # noqa: E501
            revision (str): [optional]  # noqa: E501
            show_cut_geom_only (bool): [optional]  # noqa: E501
            simplification_option (str): [optional]  # noqa: E501
            simplification_threshold (float): [optional]  # noqa: E501
            size (str): [optional]  # noqa: E501
            sketch_ids ([str]): [optional]  # noqa: E501
            standard (str): [optional]  # noqa: E501
            start_zones (str): [optional]  # noqa: E501
            template_args ([str]): [optional]  # noqa: E501
            template_document_id (str): [optional]  # noqa: E501
            template_element_id (str): [optional]  # noqa: E501
            template_name (str): [optional]  # noqa: E501
            template_version_id (str): [optional]  # noqa: E501
            template_workspace_id (str): [optional]  # noqa: E501
            titleblock (bool): [optional]  # noqa: E501
            units (str): [optional]  # noqa: E501
            views (str): [optional]  # noqa: E501
            workspace_id (str): [optional]  # noqa: E501
        """

        self._data_store = {}
        self._check_type = _check_type
        self._from_server = _from_server
        self._path_to_item = _path_to_item
        self._configuration = _configuration

        for var_name, var_value in six.iteritems(kwargs):
            if (
                var_name not in self.attribute_map
                and self._configuration is not None
                and self._configuration.discard_unknown_keys
                and self.additional_properties_type is None
            ):
                # discard variable.
                continue
            setattr(self, var_name, var_value)



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    The version of the OpenAPI document: 1.113
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import
import re  # noqa: F401
import sys  # noqa: F401

import six  # noqa: F401
import nulltype  # noqa: F401

from onshape_client.oas.model_utils import (  # noqa: F401
    ModelComposed,
    ModelNormal,
    ModelSimple,
    date,
    datetime,
    file_type,
    int,
    none_type,
    str,
    validate_get_composed_info,
)


class BTExportModelParams(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {}

    validations = {}

    additional_properties_type = None

    @staticmethod
    def openapi_types():
        """
        This must be a class method so a model may have properties that are
        of type self, this ensures that we don't create a cyclic import

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        return {
            "angle_tolerance": (float,),  # noqa: E501
            "batch_flat_patterns": (bool,),  # noqa: E501
            "chord_tolerance": (float,),  # noqa: E501
            "cloud_object_id": (str,),  # noqa: E501
            "cloud_storage_account_id": (str,),  # noqa: E501
            "configuration": (str,),  # noqa: E501
            "deep_search_for_foreign_data": (bool,),  # noqa: E501
            "destination_name": (str,),  # noqa: E501
            "document_id": (str,),  # noqa: E501
            "document_version_id": (str,),  # noqa: E501
            "element_id": (str,),  # noqa: E501
            "element_ids": (str,),  # noqa: E501
            "email_link": (bool,),  # noqa: E501
            "email_message": (str,),  # noqa: E501
            "email_subject": (str,),  # noqa: E501
            "email_to": (str,),  # noqa: E501
            "extract_to_s3": (bool,),  # noqa: E501
            "feature_ids": (str,),  # noqa: E501
            "flatten": (bool,),  # noqa: E501
            "format": (str,),  # noqa: E501
            "from_user_id": (str,),  # noqa: E501
            "grouping": (str,),  # noqa: E501
            "include_bend_centerlines": (bool,),  # noqa: E501
            "include_bend_lines": (bool,),  # noqa: E501
            "include_custom_properties": (bool,),  # noqa: E501
            "include_custom_properties_data": (bool,),  # noqa: E501
            "include_export_ids": (bool,),  # noqa: E501
            "include_foreign_data": (bool,),  # noqa: E501
            "include_items_data": (bool,),  # noqa: E501
            "include_linked_documents": (bool,),  # noqa: E501
            "include_release_management_data": (bool,),  # noqa: E501
            "include_sketches": (bool,),  # noqa: E501
            "include_std": (bool,),  # noqa: E501
            "is_parting_out": (bool,),  # noqa: E501
            "link_document_id": (str,),  # noqa: E501
            "link_document_workspace_id": (str,),  # noqa: E501
            "max_facet_width": (float,),  # noqa: E501
            "microversion": (str,),  # noqa: E501
            "min_facet_width": (float,),  # noqa: E501
            "mode": (str,),  # noqa: E501
            "part_ids": (str,),  # noqa: E501
            "part_query": (str,),  # noqa: E501
            "password": (str,),  # noqa: E501
            "password_required": (bool,),  # noqa: E501
            "resolution": (str,),  # noqa: E501
            "scale": (float,),  # noqa: E501
            "send_copy_to_me": (bool,),  # noqa: E501
            "sheet_metal_flat": (bool,),  # noqa: E501
            "splines_as_polylines": (bool,),  # noqa: E501
            "store_in_document": (bool,),  # noqa: E501
            "trigger_auto_download": (bool,),  # noqa: E501
            "units": (str,),  # noqa: E501
            "user_id": (str,),  # noqa: E501
            "valid_for_days": (int,),  # noqa: E501
            "version": (str,),  # noqa: E501
            "view": (str,),  # noqa: E501
            "workspace_id": (str,),  # noqa: E501
            "zip_single_file_output": (bool,),  # noqa: E501
        }

    @staticmethod
    def discriminator():
        return None

    attribute_map = {
        "angle_tolerance": "angleTolerance",  # noqa: E501
        "batch_flat_patterns": "batchFlatPatterns",  # noqa: E501
        "chord_tolerance": "chordTolerance",  # noqa: E501
        "cloud_object_id": "cloudObjectId",  # noqa: E501
        "cloud_storage_account_id": "cloudStorageAccountId",  # noqa: E501
        "configuration": "configuration",  # noqa: E501
        "deep_search_for_foreign_data": "deepSearchForForeignData",  # noqa: E501
        "destination_name": "destinationName",  # noqa: E501
        "document_id": "documentId",  # noqa: E501
        "document_version_id": "documentVersionId",  # noqa: E501
        "element_id": "elementId",  # noqa: E501
        "element_ids": "elementIds",  # noqa: E501
        "email_link": "emailLink",  # noqa: E501
        "email_message": "emailMessage",  # noqa: E501
        "email_subject": "emailSubject",  # noqa: E501
        "email_to": "emailTo",  # noqa: E501
        "extract_to_s3": "extractToS3",  # noqa: E501
        "feature_ids": "featureIds",  # noqa: E501
        "flatten": "flatten",  # noqa: E501
        "format": "format",  # noqa: E501
        "from_user_id": "fromUserId",  # noqa: E501
        "grouping": "grouping",  # noqa: E501
        "include_bend_centerlines": "includeBendCenterlines",  # noqa: E501
        "include_bend_lines": "includeBendLines",  # noqa: E501
        "include_custom_properties": "includeCustomProperties",  # noqa: E501
        "include_custom_properties_data": "includeCustomPropertiesData",  # noqa: E501
        "include_export_ids": "includeExportIds",  # noqa: E501
        "include_foreign_data": "includeForeignData",  # noqa: E501
        "include_items_data": "includeItemsData",  # noqa: E501
        "include_linked_documents": "includeLinkedDocuments",  # noqa: E501
        "include_release_management_data": "includeReleaseManagementData",  # noqa: E501
        "include_sketches": "includeSketches",  # noqa: E501
        "include_std": "includeStd",  # noqa: E501
        "is_parting_out": "isPartingOut",  # noqa: E501
        "link_document_id": "linkDocumentId",  # noqa: E501
        "link_document_workspace_id": "linkDocumentWorkspaceId",  # noqa: E501
        "max_facet_width": "maxFacetWidth",  # noqa: E501
        "microversion": "microversion",  # noqa: E501
        "min_facet_width": "minFacetWidth",  # noqa: E501
        "mode": "mode",  # noqa: E501
        "part_ids": "partIds",  # noqa: E501
        "part_query": "partQuery",  # noqa: E501
        "password": "password",  # noqa: E501
        "password_required": "passwordRequired",  # noqa: E501
        "resolution": "resolution",  # noqa: E501
        "scale": "scale",  # noqa: E501
        "send_copy_to_me": "sendCopyToMe",  # noqa: E501
        "sheet_metal_flat": "sheetMetalFlat",  # noqa: E501
        "splines_as_polylines": "splinesAsPolylines",  # noqa: E501
        "store_in_document": "storeInDocument",  # noqa: E501
        "trigger_auto_download": "triggerAutoDownload",  # noqa: E501
        "units": "units",  # noqa: E501
        "user_id": "userId",  # noqa: E501
        "valid_for_days": "validForDays",  # noqa: E501
        "version": "version",  # noqa: E501
        "view": "view",  # noqa: E501
        "workspace_id": "workspaceId",  # noqa: E501
        "zip_single_file_output": "zipSingleFileOutput",  # noqa: E501
    }

    @staticmethod
    def _composed_schemas():
        return None

    required_properties = set(
        [
            "_data_store",
            "_check_type",
            "_from_server",
            "_path_to_item",
            "_configuration",
        ]
    )

    def __init__(
        self,
        _check_type=True,
        _from_server=False,
        _path_to_item=(),
        _configuration=None,
        **kwargs
    ):  # noqa: E501
        """bt_export_model_params.BTExportModelParams - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _from_server (bool): True if the data is from the server
                                False if the data is from the client (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            angle_tolerance (float): [optional]  # noqa: E501
            batch_flat_patterns (bool): [optional]  # noqa: E501
            chord_tolerance (float): [optional]  # noqa: E501
            cloud_object_id (str): [optional]  # noqa: E501
            cloud_storage_account_id (str): [optional]  # noqa: E501
            configuration (str): [optional]  # noqa: E501
            deep_search_for_foreign_data (bool): [optional]  # noqa: E501
            destination_name (str): [optional]  # noqa: E501
            document_id (str): [optional]  # noqa: E501
            document_version_id (str): [optional]  # noqa: E501
            element_id (str): [optional]  # noqa: E501
            element_ids (str): [optional]  # noqa: E501
            email_link (bool): [optional]  # noqa: E501
            email_message (str): [optional]  # noqa: E501
            email_subject (str): [optional]  # noqa: E501
            email_to (str): [optional]  # noqa: E501
            extract_to_s3 (bool): [optional]  # noqa: E501
            feature_ids (str): [optional]  # noqa: E501
            flatten (bool): [optional]  # noqa: E501
            format (str): [optional]  # noqa: E501
            from_user_id (str): [optional]  # noqa: E501
            grouping (str): [optional]  # noqa: E501
            include_bend_centerlines (bool): [optional]  # noqa: E501
            include_bend_lines (bool): [optional]  # noqa: E501
            include_custom_properties (bool): [optional]  # noqa: E501
            include_custom_properties_data (bool): [optional]  # noqa: E501
            include_export_ids (bool): [optional]  # noqa: E501
            include_foreign_data (bool): [optional]  # noqa: E501
            include_items_data (bool): [optional]  # noqa: E501
            include_linked_documents (bool): [optional]  # noqa: E501
            include_release_management_data (bool): [optional]  # noqa: E501
            include_sketches (bool): [optional]  # noqa: E501
            include_std (bool): [optional]  # noqa: E501
            is_parting_out (bool): [optional]  # noqa: E501
            link_document_id (str): [optional]  # noqa: E501
            link_document_workspace_id (str): [optional]  # noqa: E501
            max_facet_width (float): [optional]  # noqa: E501
            microversion (str): [optional]  # noqa: E501
            min_facet_width (float): [optional]  # noqa: E501
            mode (str): [optional]  # noqa: E501
            part_ids (str): [optional]  # noqa: E501
            part_query (str): [optional]  # noqa: E501
            password (str): [optional]  # noqa: E501
            password_required (bool): [optional]  # noqa: E501
            resolution (str): [optional]  # noqa: E501
            scale (float): [optional]  # noqa: E501
            send_copy_to_me (bool): [optional]  # noqa: E501
            sheet_metal_flat (bool): [optional]  # noqa: E501
            splines_as_polylines (bool): [optional]  # noqa: E501
            store_in_document (bool): [optional]  # noqa: E501
            trigger_auto_download (bool): [optional]  # noqa: E501
            units (str): [optional]  # noqa: E501
            user_id (str): [optional]  # noqa: E501
            valid_for_days (int): [optional]  # noqa: E501
            version (str): [optional]  # noqa: E501
            view (str): [optional]  # noqa: E501
            workspace_id (str): [optional]  # noqa: E501
            zip_single_file_output (bool): [optional]  # noqa: E501
        """

        self._data_store = {}
        self._check_type = _check_type
        self._from_server = _from_server
        self._path_to_item = _path_to_item
        self._configuration = _configuration

        for var_name, var_value in six.iteritems(kwargs):
            if (
                var_name not in self.attribute_map
                and self._configuration is not None
                and self._configuration.discard_unknown_keys
                and self.additional_properties_type is None
            ):
                # discard variable.
                continue
            setattr(self, var_name, var_value)



import json
import string
import time
from urllib.parse import urlparse, parse_qs

import pytest
import random
from onshape_client.oas import BTCopyDocumentParams, BTDocumentSearchParams
from onshape_client.onshape_url import OnshapeElement


def test_copy_workspace(client, new_document):
    NEW_DOCUMENT_NAME = "My copied document"
    new_doc_info = client.documents_api.copy_workspace(
        did=new_document.did,
        wid=new_document.wvmid,
        bt_copy_document_params=BTCopyDocumentParams(
            new_name=NEW_DOCUMENT_NAME, is_public=True
        ),
    )
    assert new_doc_info.new_document_name == NEW_DOCUMENT_NAME


def test_get_documents_simple(client):
    unique_name = "".join(random.choices(string.ascii_uppercase + string.digits, k=8))
    doc = OnshapeElement.create(unique_name)
    time.sleep(5)
    documents = client.documents_api.get_documents(q=unique_name)
    doc.delete()
    assert len(documents["items"]) == 1


@pytest.mark.skip(reason="Not done")
def test_create_company_owned_document(client):
    """Try creating a company-owned document"""
    doc = client.documents_api.create_document(
        bt_document_params=BTCopyDocumentParams(
            owner_type_index=1, new_name="Test enterprise owned doc", owner_id=""
        )
    )


@pytest.mark.skip(reason="Example code for paginated document search")
def test_advanced_document_search(client):
    SEARCH_TERM = "3DBK Maker Frames"
    response = client.documents_api.search(
        bt_document_search_params=BTDocumentSearchParams(
            found_in="ALL", raw_query=f"_all:{SEARCH_TERM} type:all"
        ),
        _preload_content=False,
    )
    result = json.loads(response.data)
    doc_ids = []
    while result["next"]:
        next = urlparse(result["next"])
        offset = int(parse_qs(next.query)["offset"][0])
        doc_ids = doc_ids + [
            item["id"]
            for item in result["items"]
            if any(
                [search_hit["name"] == SEARCH_TERM for search_hit in item["searchHits"]]
            )
        ]
        result = json.loads(
            client.documents_api.search(
                bt_document_search_params=BTDocumentSearchParams(
                    found_in="ALL",
                    raw_query=f'_all:"{SEARCH_TERM}" type:all',
                    offset=offset,
                ),
                _preload_content=False,
            ).data
        )
    assert len(doc_ids) > 1



import pytest
from onshape_client.client import Client
from onshape_client.oas import BTTranslateFormatParams
from onshape_client.onshape_url import OnshapeElement
from pathlib import Path


@pytest.mark.parametrize(
    ["element", "format_name"],
    [
        ("drawing_configurable_cube", "PDF"),
        ("asm_three_axes", "PARASOLID"),
        ("ps_configurable_cube", "PARASOLID"),
        ("ps_configurable_cube", "STEP"),
    ],
    indirect=["element"],
)
def test_export(
    client: Client,
    element: OnshapeElement,
    tmp_dir,
    format_name,
    translator_format_to_extension,
):
    file_name = f"{element.name}.{translator_format_to_extension[format_name]}"
    element.export_file(
        tmp_dir / file_name,
        BTTranslateFormatParams(
            destination_name=file_name,
            format_name=format_name,
            store_in_document=False,
        ),
    )


@pytest.mark.parametrize(
    "element", ["ps_configurable_cube"], indirect=True,
)
def test_export_ps_stl(client: Client, element: OnshapeElement, tmp_dir: Path):
    file = tmp_dir / f"{element.name}.stl"
    response = client.part_studios_api.export_stl1(
        element.did, element.wvm, element.wvmid, element.eid, _preload_content=False
    )
    with file.open(mode="wb") as f:
        f.write(response.data)
    assert file.stat().st_size > 1000



"""This is for storing all the Onshape-specific functionality added so it is all in one place"""
import re

from onshape_client.compatible_imports import unquote


def decode_file_name_from_header(content_disposition_header):
    filename_potential = re.search(
        r"""filename\*?=([^\s]+)""", content_disposition_header
    )
    if "UTF-8" in filename_potential.group(1):
        filename_potential = re.search(
            r"""UTF-8[^\d\%]*([^\'\s]+)""", content_disposition_header
        )
        filename = unquote(filename_potential.group(1))
    else:
        filename = filename_potential.group(1).replace('"', "").replace("'", "")
    return filename


def normalize_response_data(response_data):
    if isinstance(response_data, bytes):
        return response_data
    else:
        return response_data.encode()



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    The version of the OpenAPI document: 1.113
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import
import re  # noqa: F401
import sys  # noqa: F401

import six  # noqa: F401

from onshape_client.oas.model_utils import (  # noqa: F401
    ModelComposed,
    ModelNormal,
    ModelSimple,
    date,
    datetime,
    file_type,
    int,
    none_type,
    str,
    validate_get_composed_info,
)

try:
    from onshape_client.oas.models import discriminator
except ImportError:
    discriminator = sys.modules["onshape_client.oas.models.discriminator"]
try:
    from onshape_client.oas.models import external_documentation
except ImportError:
    external_documentation = sys.modules[
        "onshape_client.oas.models.external_documentation"
    ]
try:
    from onshape_client.oas.models import xml
except ImportError:
    xml = sys.modules["onshape_client.oas.models.xml"]


class Schema(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {}

    validations = {}

    additional_properties_type = None

    @staticmethod
    def openapi_types():
        """
        This must be a class method so a model may have properties that are
        of type self, this ensures that we don't create a cyclic import

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        return {
            "additional_properties": (
                bool,
                date,
                datetime,
                dict,
                float,
                int,
                list,
                str,
            ),  # noqa: E501
            "default": (
                bool,
                date,
                datetime,
                dict,
                float,
                int,
                list,
                str,
            ),  # noqa: E501
            "deprecated": (bool,),  # noqa: E501
            "description": (str,),  # noqa: E501
            "discriminator": (discriminator.Discriminator,),  # noqa: E501
            "enum": (
                [bool, date, datetime, dict, float, int, list, str],
            ),  # noqa: E501
            "example": (
                bool,
                date,
                datetime,
                dict,
                float,
                int,
                list,
                str,
            ),  # noqa: E501
            "exclusive_maximum": (bool,),  # noqa: E501
            "exclusive_minimum": (bool,),  # noqa: E501
            "extensions": (
                {str: (bool, date, datetime, dict, float, int, list, str,)},
            ),  # noqa: E501
            "external_docs": (
                external_documentation.ExternalDocumentation,
            ),  # noqa: E501
            "format": (str,),  # noqa: E501
            "getref": (str,),  # noqa: E501
            "max_items": (int,),  # noqa: E501
            "max_length": (int,),  # noqa: E501
            "max_properties": (int,),  # noqa: E501
            "maximum": (float,),  # noqa: E501
            "min_items": (int,),  # noqa: E501
            "min_length": (int,),  # noqa: E501
            "min_properties": (int,),  # noqa: E501
            "minimum": (float,),  # noqa: E501
            "multiple_of": (float,),  # noqa: E501
            "_not": (Schema,),  # noqa: E501
            "nullable": (bool,),  # noqa: E501
            "pattern": (str,),  # noqa: E501
            "properties": ({str: (Schema,)},),  # noqa: E501
            "read_only": (bool,),  # noqa: E501
            "required": ([str],),  # noqa: E501
            "title": (str,),  # noqa: E501
            "type": (str,),  # noqa: E501
            "unique_items": (bool,),  # noqa: E501
            "write_only": (bool,),  # noqa: E501
            "xml": (xml.XML,),  # noqa: E501
        }

    @staticmethod
    def discriminator():
        return None

    attribute_map = {
        "additional_properties": "additionalProperties",  # noqa: E501
        "default": "default",  # noqa: E501
        "deprecated": "deprecated",  # noqa: E501
        "description": "description",  # noqa: E501
        "discriminator": "discriminator",  # noqa: E501
        "enum": "enum",  # noqa: E501
        "example": "example",  # noqa: E501
        "exclusive_maximum": "exclusiveMaximum",  # noqa: E501
        "exclusive_minimum": "exclusiveMinimum",  # noqa: E501
        "extensions": "extensions",  # noqa: E501
        "external_docs": "externalDocs",  # noqa: E501
        "format": "format",  # noqa: E501
        "getref": "get$ref",  # noqa: E501
        "max_items": "maxItems",  # noqa: E501
        "max_length": "maxLength",  # noqa: E501
        "max_properties": "maxProperties",  # noqa: E501
        "maximum": "maximum",  # noqa: E501
        "min_items": "minItems",  # noqa: E501
        "min_length": "minLength",  # noqa: E501
        "min_properties": "minProperties",  # noqa: E501
        "minimum": "minimum",  # noqa: E501
        "multiple_of": "multipleOf",  # noqa: E501
        "_not": "not",  # noqa: E501
        "nullable": "nullable",  # noqa: E501
        "pattern": "pattern",  # noqa: E501
        "properties": "properties",  # noqa: E501
        "read_only": "readOnly",  # noqa: E501
        "required": "required",  # noqa: E501
        "title": "title",  # noqa: E501
        "type": "type",  # noqa: E501
        "unique_items": "uniqueItems",  # noqa: E501
        "write_only": "writeOnly",  # noqa: E501
        "xml": "xml",  # noqa: E501
    }

    @staticmethod
    def _composed_schemas():
        return None

    required_properties = set(
        [
            "_data_store",
            "_check_type",
            "_from_server",
            "_path_to_item",
            "_configuration",
        ]
    )

    def __init__(
        self,
        _check_type=True,
        _from_server=False,
        _path_to_item=(),
        _configuration=None,
        **kwargs
    ):  # noqa: E501
        """schema.Schema - a model defined in OpenAPI


        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _from_server (bool): True if the data is from the server
                                False if the data is from the client (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            additional_properties (bool, date, datetime, dict, float, int, list, str): [optional]  # noqa: E501
            default (bool, date, datetime, dict, float, int, list, str): [optional]  # noqa: E501
            deprecated (bool): [optional]  # noqa: E501
            description (str): [optional]  # noqa: E501
            discriminator (discriminator.Discriminator): [optional]  # noqa: E501
            enum ([bool, date, datetime, dict, float, int, list, str]): [optional]  # noqa: E501
            example (bool, date, datetime, dict, float, int, list, str): [optional]  # noqa: E501
            exclusive_maximum (bool): [optional]  # noqa: E501
            exclusive_minimum (bool): [optional]  # noqa: E501
            extensions ({str: (bool, date, datetime, dict, float, int, list, str,)}): [optional]  # noqa: E501
            external_docs (external_documentation.ExternalDocumentation): [optional]  # noqa: E501
            format (str): [optional]  # noqa: E501
            getref (str): [optional]  # noqa: E501
            max_items (int): [optional]  # noqa: E501
            max_length (int): [optional]  # noqa: E501
            max_properties (int): [optional]  # noqa: E501
            maximum (float): [optional]  # noqa: E501
            min_items (int): [optional]  # noqa: E501
            min_length (int): [optional]  # noqa: E501
            min_properties (int): [optional]  # noqa: E501
            minimum (float): [optional]  # noqa: E501
            multiple_of (float): [optional]  # noqa: E501
            _not (Schema): [optional]  # noqa: E501
            nullable (bool): [optional]  # noqa: E501
            pattern (str): [optional]  # noqa: E501
            properties ({str: (Schema,)}): [optional]  # noqa: E501
            read_only (bool): [optional]  # noqa: E501
            required ([str]): [optional]  # noqa: E501
            title (str): [optional]  # noqa: E501
            type (str): [optional]  # noqa: E501
            unique_items (bool): [optional]  # noqa: E501
            write_only (bool): [optional]  # noqa: E501
            xml (xml.XML): [optional]  # noqa: E501
        """

        self._data_store = {}
        self._check_type = _check_type
        self._from_server = _from_server
        self._path_to_item = _path_to_item
        self._configuration = _configuration

        for var_name, var_value in six.iteritems(kwargs):
            setattr(self, var_name, var_value)



"""This includes all the test-wide fixtures. Element fixtures that return an OnshapeElement are prefixed with el_
"""

import json
from collections import OrderedDict
from datetime import datetime
from pathlib import Path

import pytest
from onshape_client.client import Client
from onshape_client.oas import BTCopyDocumentParams
from onshape_client.onshape_url import OnshapeElement
from ruamel.yaml import YAML

collect_ignore = ["setup.py"]


@pytest.fixture(scope="session")
def element_bank(client, assets):
    yaml = YAML()
    urls_by_stack = yaml.load((assets / "urls.yaml").open())
    stack_key = client.stack_key if client.stack_key else "onshape_client_test"
    try:
        return urls_by_stack[stack_key]
    except KeyError as e:
        raise KeyError(
            f"Cannot find element bank for stack {client.stack_key}. Please instantiate the bank manually by uploading "
            f"the needed docs to the stack and updating the url.yaml file."
        )


@pytest.fixture(scope="session")
def element(request, client, element_bank) -> OnshapeElement:
    """ Get a particular element from the bank by parametrizing the test with: `@pytest.mark.parametrize('assembly',
    ['three_axes'], indirect=True)`
    :param request:
    :param client:
    :return: OnshapeElement
    """
    element_key = request.param
    try:
        return OnshapeElement(element_bank[element_key])
    except KeyError as e:
        raise KeyError(f"Cannot find '{element_key}' in the element bank.")


@pytest.fixture(scope="session")
def client() -> Client:
    """Client needed to make API calls."""
    try:
        client = Client.get_client()
    except Exception as e:
        client = Client(stack_key="onshape_client_test")
    return client


@pytest.fixture
def translator_formats(client):
    return client.translation_api.get_all_translator_formats()


@pytest.fixture
def translator_format_to_extension():
    return {"PARASOLID": "x_t", "PDF": "pdf", "STL": "stl", "STEP": "stp"}


@pytest.fixture(scope="session")
def test_dir():
    return Path(__file__).parent


@pytest.fixture(scope="session")
def assets(test_dir):
    """Returns the general test assets folder."""
    return test_dir / "assets"


@pytest.fixture(scope="session")
def json_assets(assets):
    """Returns the JSON assets folder."""
    return assets / "json"


@pytest.fixture(scope="session")
def translated_geometry(assets):
    """Returns the translated geometry assets folder."""
    return assets / "translated_geometry"


@pytest.fixture(scope="session")
def tmp_dir(test_dir):
    """Returns a temporary directory Path object."""
    tmp_dir = test_dir / "tmp"
    if not tmp_dir.is_dir():
        tmp_dir.mkdir()
    return tmp_dir


@pytest.fixture
def new_document(request, client, name_factory):
    """Returns a blank new document."""
    doc = OnshapeElement.create(name_factory())
    yield doc
    doc.delete()


@pytest.fixture
def part_studio(new_document):
    return new_document.part_studios[0]


@pytest.fixture
def new_copied_document_factory(client, name_factory):
    """Factory to copy elements that cleans up created elements when done."""
    created_docs = []

    def copy_workspace(copied_from):
        params = BTCopyDocumentParams(new_name=name_factory(), is_public=True)
        doc = client.documents_api.copy_workspace(
            copied_from.did,
            copied_from.default_workspace,
            bt_copy_document_params=params,
            _preload_content=False,
        )
        doc = json.loads(doc.data.decode("utf-8"))
        doc = OnshapeElement.create_from_ids(
            did=doc["newDocumentId"], wvm="w", wvmid=doc["newWorkspaceId"]
        )
        created_docs.append(doc)
        return doc

    yield copy_workspace
    for d in created_docs:
        d.delete()


@pytest.fixture(scope="session")
def name_factory():
    """Factory to generate a doc name."""

    def name_factory():
        return f"Python Onshape Client Doc {datetime.now()}"

    return name_factory



import pytest
from onshape_client.client import OAuthAuthorizationMethods


@pytest.mark.skip(reason="no way of currently testing this")
def test_manual_oauth_flow(client):
    client.oauth_authorization_method = OAuthAuthorizationMethods.MANUAL_FLOW
    # save refresh token then put in later.
    refresh_token = client.refresh_token
    client.refresh_token = ""
    client.configuration.access_token = ""
    try:
        client.documents_api.get_documents()
    except Exception as e:
        client.refresh_token = refresh_token
        client.documents_api.get_documents()



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    The version of the OpenAPI document: 1.113
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import
import re  # noqa: F401
import sys  # noqa: F401

import six  # noqa: F401
import nulltype  # noqa: F401

from onshape_client.oas.model_utils import (  # noqa: F401
    ModelComposed,
    ModelNormal,
    ModelSimple,
    date,
    datetime,
    file_type,
    int,
    none_type,
    str,
    validate_get_composed_info,
)

try:
    from onshape_client.oas.models import btm_parameter1
except ImportError:
    btm_parameter1 = sys.modules["onshape_client.oas.models.btm_parameter1"]


class BTMSketchConstraint2(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ("constraint_type",): {
            "NONE": "NONE",
            "COINCIDENT": "COINCIDENT",
            "PARALLEL": "PARALLEL",
            "VERTICAL": "VERTICAL",
            "HORIZONTAL": "HORIZONTAL",
            "PERPENDICULAR": "PERPENDICULAR",
            "CONCENTRIC": "CONCENTRIC",
            "MIRROR": "MIRROR",
            "MIDPOINT": "MIDPOINT",
            "TANGENT": "TANGENT",
            "EQUAL": "EQUAL",
            "LENGTH": "LENGTH",
            "DISTANCE": "DISTANCE",
            "ANGLE": "ANGLE",
            "RADIUS": "RADIUS",
            "NORMAL": "NORMAL",
            "FIX": "FIX",
            "PROJECTED": "PROJECTED",
            "OFFSET": "OFFSET",
            "CIRCULAR_PATTERN": "CIRCULAR_PATTERN",
            "PIERCE": "PIERCE",
            "LINEAR_PATTERN": "LINEAR_PATTERN",
            "MAJOR_DIAMETER": "MAJOR_DIAMETER",
            "MINOR_DIAMETER": "MINOR_DIAMETER",
            "QUADRANT": "QUADRANT",
            "DIAMETER": "DIAMETER",
            "SILHOUETTED": "SILHOUETTED",
            "CENTERLINE_DIMENSION": "CENTERLINE_DIMENSION",
            "INTERSECTED": "INTERSECTED",
            "RHO": "RHO",
            "EQUAL_CURVATURE": "EQUAL_CURVATURE",
            "UNKNOWN": "UNKNOWN",
        },
    }

    validations = {}

    additional_properties_type = None

    @staticmethod
    def openapi_types():
        """
        This must be a class method so a model may have properties that are
        of type self, this ensures that we don't create a cyclic import

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        return {
            "bt_type": (str,),  # noqa: E501
            "constraint_type": (str,),  # noqa: E501
            "driven_dimension": (bool,),  # noqa: E501
            "entity_id": (str,),  # noqa: E501
            "entity_id_and_replace_in_dependent_fields": (str,),  # noqa: E501
            "has_offset_data1": (bool,),  # noqa: E501
            "has_offset_data2": (bool,),  # noqa: E501
            "has_pierce_parameter": (bool,),  # noqa: E501
            "help_parameters": ([float],),  # noqa: E501
            "import_microversion": (str,),  # noqa: E501
            "namespace": (str,),  # noqa: E501
            "node_id": (str,),  # noqa: E501
            "offset_distance1": (float,),  # noqa: E501
            "offset_distance2": (float,),  # noqa: E501
            "offset_orientation1": (bool,),  # noqa: E501
            "offset_orientation2": (bool,),  # noqa: E501
            "parameters": ([btm_parameter1.BTMParameter1],),  # noqa: E501
            "pierce_parameter": (float,),  # noqa: E501
        }

    @staticmethod
    def discriminator():
        return None

    attribute_map = {
        "bt_type": "btType",  # noqa: E501
        "constraint_type": "constraintType",  # noqa: E501
        "driven_dimension": "drivenDimension",  # noqa: E501
        "entity_id": "entityId",  # noqa: E501
        "entity_id_and_replace_in_dependent_fields": "entityIdAndReplaceInDependentFields",  # noqa: E501
        "has_offset_data1": "hasOffsetData1",  # noqa: E501
        "has_offset_data2": "hasOffsetData2",  # noqa: E501
        "has_pierce_parameter": "hasPierceParameter",  # noqa: E501
        "help_parameters": "helpParameters",  # noqa: E501
        "import_microversion": "importMicroversion",  # noqa: E501
        "namespace": "namespace",  # noqa: E501
        "node_id": "nodeId",  # noqa: E501
        "offset_distance1": "offsetDistance1",  # noqa: E501
        "offset_distance2": "offsetDistance2",  # noqa: E501
        "offset_orientation1": "offsetOrientation1",  # noqa: E501
        "offset_orientation2": "offsetOrientation2",  # noqa: E501
        "parameters": "parameters",  # noqa: E501
        "pierce_parameter": "pierceParameter",  # noqa: E501
    }

    @staticmethod
    def _composed_schemas():
        return None

    required_properties = set(
        [
            "_data_store",
            "_check_type",
            "_from_server",
            "_path_to_item",
            "_configuration",
        ]
    )

    def __init__(
        self,
        _check_type=True,
        _from_server=False,
        _path_to_item=(),
        _configuration=None,
        **kwargs
    ):  # noqa: E501
        """btm_sketch_constraint2.BTMSketchConstraint2 - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _from_server (bool): True if the data is from the server
                                False if the data is from the client (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            bt_type (str): [optional]  # noqa: E501
            constraint_type (str): [optional]  # noqa: E501
            driven_dimension (bool): [optional]  # noqa: E501
            entity_id (str): [optional]  # noqa: E501
            entity_id_and_replace_in_dependent_fields (str): [optional]  # noqa: E501
            has_offset_data1 (bool): [optional]  # noqa: E501
            has_offset_data2 (bool): [optional]  # noqa: E501
            has_pierce_parameter (bool): [optional]  # noqa: E501
            help_parameters ([float]): [optional]  # noqa: E501
            import_microversion (str): [optional]  # noqa: E501
            namespace (str): [optional]  # noqa: E501
            node_id (str): [optional]  # noqa: E501
            offset_distance1 (float): [optional]  # noqa: E501
            offset_distance2 (float): [optional]  # noqa: E501
            offset_orientation1 (bool): [optional]  # noqa: E501
            offset_orientation2 (bool): [optional]  # noqa: E501
            parameters ([btm_parameter1.BTMParameter1]): [optional]  # noqa: E501
            pierce_parameter (float): [optional]  # noqa: E501
        """

        self._data_store = {}
        self._check_type = _check_type
        self._from_server = _from_server
        self._path_to_item = _path_to_item
        self._configuration = _configuration

        for var_name, var_value in six.iteritems(kwargs):
            if (
                var_name not in self.attribute_map
                and self._configuration is not None
                and self._configuration.discard_unknown_keys
                and self.additional_properties_type is None
            ):
                # discard variable.
                continue
            setattr(self, var_name, var_value)



import pytest
from onshape_client import OnshapeElement
from pathlib import Path


@pytest.mark.parametrize("file", ["Cube.x_t", "Cube.step", "Cube.stl"])
def test_import_file(new_document, client, translated_geometry, file):
    imported_ps = new_document.import_file(
        translated_geometry / file, allow_faulty_parts=True
    )
    response = client.parts_api.get_mass_properties(
        imported_ps.did,
        imported_ps.wvm,
        imported_ps.wvmid,
        imported_ps.eid,
        partid="JFD",
    )
    assert response.bodies["JFD"].volume[0] > 0


def test_batch_import_file(client, translated_geometry):
    """Upload all files in the translated_geometry folder to each new document, and get a mapping from file name to
    mass."""
    threads = []

    def upload_then_get_mass(file: Path):
        new_doc = OnshapeElement.create(file.name)
        imported_ps = new_doc.import_file(file, allow_faulty_parts=True)
        return imported_ps.mass_properties.bodies["-all-"].volume[0]

    for file in translated_geometry.iterdir():
        threads.append(client.api_client.pool.apply_async(upload_then_get_mass, [file]))

    result = {}

    for thread, file in zip(threads, translated_geometry.iterdir()):
        result[file.name] = round(thread.get(), 7)

    assert result == {
        "Many Many Cubes.step": 1.0,
        "Cube.x_t": 0.0004425,
        "Cube.step": 1.0,
        "Cube.stl": 61023.5329527,
    }
